@@ -540,24 +540,34 @@ static struct libmnt_fs *lookup(char *path)
 }
 
 /* Find fsck program for a given fs type. */
-static char *find_fsck(const char *type)
+static int find_fsck(const char *type, char **progpath)
 {
 	char *s;
 	const char *tpl;
-	static char prog[256];
+	char *prog = NULL;
 	char *p = xstrdup(fsck_path);
+	int rc;
 
 	/* Are we looking for a program or just a type? */
 	tpl = (strncmp(type, "fsck.", 5) ? "%s/fsck.%s" : "%s/%s");
 
 	for(s = strtok(p, ":"); s; s = strtok(NULL, ":")) {
-		sprintf(prog, tpl, s, type);
+		xasprintf(&prog, tpl, s, type);
 		if (access(prog, X_OK) == 0)
 			break;
+		free(prog);
+		prog = NULL;
 	}
-	free(p);
 
-	return(s ? prog : NULL);
+	free(p);
+	rc = prog ? 1 : 0;
+
+	if (progpath)
+		*progpath = prog;
+	else
+		free(prog);
+
+	return rc;
 }
 
 static int progress_active(void)
@@ -885,7 +895,7 @@ static int wait_many(int flags)
  */
 static int fsck_device(struct libmnt_fs *fs, int interactive)
 {
-	char progname[80], *progpath;
+	char *progname, *progpath;
 	const char *type;
 	int retval;
 
@@ -902,9 +912,10 @@ static int fsck_device(struct libmnt_fs *fs, int interactive)
 	else
 		type = DEFAULT_FSTYPE;
 
-	sprintf(progname, "fsck.%s", type);
-	progpath = find_fsck(progname);
-	if (progpath == NULL) {
+	xasprintf(&progname, "fsck.%s", type);
+
+	if (!find_fsck(progname, &progpath)) {
+		free(progname);
 		if (fs_check_required(type)) {
 			retval = ENOENT;
 			goto err;
@@ -914,6 +925,8 @@ static int fsck_device(struct libmnt_fs *fs, int interactive)
 
 	num_running++;
 	retval = execute(progname, progpath, type, fs, interactive);
+	free(progname);
+	free(progpath);
 	if (retval) {
 		num_running--;
 		goto err;
@@ -1148,7 +1161,7 @@ static int ignore(struct libmnt_fs *fs)
 
 
 	/* See if the <fsck.fs> program is available. */
-	if (find_fsck(type) == NULL) {
+	if (!find_fsck(type, NULL)) {
 		if (fs_check_required(type))
 			warnx(_("cannot check %s: fsck.%s not found"),
 				fs_get_device(fs), type);
@@ -1437,10 +1450,8 @@ static void parse_argv(int argc, char *argv[])
 		/* the only two longopts to satisfy UL standards */
 		if (!opts_for_fsck && !strcmp(arg, "--help"))
 			usage();
-		if (!opts_for_fsck && !strcmp(arg, "--version")) {
-			printf(UTIL_LINUX_VERSION);
-			exit(FSCK_EX_OK);
-		}
+		if (!opts_for_fsck && !strcmp(arg, "--version"))
+			print_version(FSCK_EX_OK);
 
 		if ((arg[0] == '/' && !opts_for_fsck) || strchr(arg, '=')) {
 			if (num_devices >= MAX_DEVICES)
@@ -1608,7 +1619,7 @@ int main(int argc, char *argv[])
 	setlocale(LC_CTYPE, "");
 	bindtextdomain(PACKAGE, LOCALEDIR);
 	textdomain(PACKAGE);
-	atexit(close_stdout);
+	close_stdout_atexit();
 
 	strutils_set_exitcode(FSCK_EX_USAGE);
 	mnt_init_debug(0);		/* init libmount debug mask */
