@@ -177,7 +177,7 @@ int main(int argc, char **argv)
 	setlocale(LC_ALL, "");
 	bindtextdomain(PACKAGE, LOCALEDIR);
 	textdomain(PACKAGE);
-	atexit(close_stdout);
+	close_stdout_atexit();
 
 	while ((opt = getopt_long(argc, argv, "d:xVh", longopts, NULL)) != -1) {
 		switch (opt) {
@@ -190,8 +190,7 @@ int main(int argc, char **argv)
 			xflag = 1;
 			break;
 		case 'V':
-			printf(UTIL_LINUX_VERSION);
-			return EXIT_SUCCESS;
+			print_version(EXIT_SUCCESS);
 		case 'h':
 			usage();
 		default:
