@@ -82,7 +82,7 @@ static void init_signature_page(struct mkswap_control *ctl)
 		if (ctl->user_pagesize != kernel_pagesize)
 			warnx(_("Using user-specified page size %d, "
 				"instead of the system value %d"),
-				ctl->pagesize, kernel_pagesize);
+				ctl->user_pagesize, kernel_pagesize);
 		ctl->pagesize = ctl->user_pagesize;
 	} else
 		ctl->pagesize = kernel_pagesize;
@@ -259,6 +259,8 @@ static void wipe_device(struct mkswap_control *ctl)
 	blkid_probe pr = NULL;
 #endif
 	if (!ctl->force) {
+		const char *v = NULL;
+
 		if (lseek(ctl->fd, 0, SEEK_SET) != 0)
 			errx(EXIT_FAILURE, _("unable to rewind swap-device"));
 
@@ -268,9 +270,8 @@ static void wipe_device(struct mkswap_control *ctl)
 		blkid_probe_enable_superblocks(pr, 0);
 
 		if (blkid_do_fullprobe(pr) == 0 &&
-		    blkid_probe_lookup_value(pr, "PTTYPE",
-				(const char **) &type, NULL) == 0 && type) {
-			type = xstrdup(type);
+		    blkid_probe_lookup_value(pr, "PTTYPE", &v, NULL) == 0 && v) {
+			type = xstrdup(v);
 			zap = 0;
 		}
 #else
@@ -368,7 +369,7 @@ int main(int argc, char **argv)
 	setlocale(LC_ALL, "");
 	bindtextdomain(PACKAGE, LOCALEDIR);
 	textdomain(PACKAGE);
-	atexit(close_stdout);
+	close_stdout_atexit();
 
 	while((c = getopt_long(argc, argv, "cfp:L:v:U:Vh", longopts, NULL)) != -1) {
 		switch (c) {
@@ -399,8 +400,7 @@ int main(int argc, char **argv)
 #endif
 			break;
 		case 'V':
-			printf(UTIL_LINUX_VERSION);
-			exit(EXIT_SUCCESS);
+			print_version(EXIT_SUCCESS);
 		case 'h':
 			usage();
 		default:
