@@ -133,7 +133,7 @@ static int get_user_reply(const char *prompt, char *buf, size_t bufsz)
 		p = readline(prompt);
 		if (!p)
 			return 1;
-		memcpy(buf, p, bufsz);
+		xstrncpy(buf, p, bufsz);
 		free(p);
 	} else
 #endif
@@ -190,7 +190,7 @@ static int ask_callback(struct fdisk_context *cxt __attribute__((__unused__)),
 		break;
 	case FDISK_ASKTYPE_YESNO:
 	{
-		char buf[BUFSIZ];
+		char buf[BUFSIZ] = { '\0' };
 		fputc('\n', stdout);
 		do {
 			int x;
@@ -824,10 +824,15 @@ static int command_activate(struct sfdisk *sf, int argc, char **argv)
 		err(EXIT_FAILURE, _("cannot open %s"), devname);
 
 	if (fdisk_is_label(sf->cxt, GPT)) {
+		if (fdisk_gpt_is_hybrid(sf->cxt))
+			errx(EXIT_FAILURE, _("toggle boot flags is unsupported for Hybrid GPT/MBR"));
+
 		/* Switch from GPT to PMBR */
 		sf->cxt = fdisk_new_nested_context(sf->cxt, "dos");
 		if (!sf->cxt)
 			err(EXIT_FAILURE, _("cannot switch to PMBR"));
+		fdisk_info(sf->cxt, _("Activation is unsupported for GPT -- entering nested PMBR."));
+
 	} else if (!fdisk_is_label(sf->cxt, DOS))
 		errx(EXIT_FAILURE, _("toggle boot flags is supported for MBR or PMBR only"));
 
@@ -1717,7 +1722,7 @@ static int command_fdisk(struct sfdisk *sf, int argc, char **argv)
 
 		nparts = fdisk_table_get_nents(tb);
 		if (nparts) {
-			size_t cur_partno;
+			size_t cur_partno = (size_t) -1;
 			struct fdisk_partition *pa = fdisk_table_get_partition(tb, nparts - 1);
 
 			assert(pa);
@@ -1987,7 +1992,7 @@ int main(int argc, char *argv[])
 	setlocale(LC_ALL, "");
 	bindtextdomain(PACKAGE, LOCALEDIR);
 	textdomain(PACKAGE);
-	atexit(close_stdout);
+	close_stdout_atexit();
 
 	while ((c = getopt_long(argc, argv, "aAbcdfFgGhJlLo:O:nN:qrsTu:vVX:Y:w:W:",
 					longopts, &longidx)) != -1) {
@@ -2072,9 +2077,7 @@ int main(int argc, char *argv[])
 				errx(EXIT_FAILURE, _("unsupported unit '%c'"), *optarg);
 			break;
 		case 'v':
-			printf(_("%s from %s\n"), program_invocation_short_name,
-						  PACKAGE_STRING);
-			return EXIT_SUCCESS;
+			print_version(EXIT_SUCCESS);
 		case 'V':
 			sf->verify = 1;
 			break;
