@@ -19,14 +19,15 @@
 # define XALLOC_EXIT_CODE EXIT_FAILURE
 #endif
 
-static inline void __err_oom(const char *file, unsigned int line)
+static inline void __attribute__((__noreturn__))
+__err_oom(const char *file, unsigned int line)
 {
 	err(XALLOC_EXIT_CODE, "%s: %u: cannot allocate memory", file, line);
 }
 
 #define err_oom()	__err_oom(__FILE__, __LINE__)
 
-static inline __ul_alloc_size(1)
+static inline __ul_alloc_size(1) __ul_returns_nonnull
 void *xmalloc(const size_t size)
 {
         void *ret = malloc(size);
@@ -36,7 +37,7 @@ void *xmalloc(const size_t size)
         return ret;
 }
 
-static inline __ul_alloc_size(2)
+static inline __ul_alloc_size(2) __ul_returns_nonnull
 void *xrealloc(void *ptr, const size_t size)
 {
         void *ret = realloc(ptr, size);
@@ -46,7 +47,7 @@ void *xrealloc(void *ptr, const size_t size)
         return ret;
 }
 
-static inline __ul_calloc_size(1, 2)
+static inline __ul_calloc_size(1, 2) __ul_returns_nonnull
 void *xcalloc(const size_t nelems, const size_t size)
 {
         void *ret = calloc(nelems, size);
@@ -56,7 +57,8 @@ void *xcalloc(const size_t nelems, const size_t size)
         return ret;
 }
 
-static inline char __attribute__((warn_unused_result)) *xstrdup(const char *str)
+static inline char __attribute__((warn_unused_result)) __ul_returns_nonnull
+*xstrdup(const char *str)
 {
         char *ret;
 
@@ -70,7 +72,8 @@ static inline char __attribute__((warn_unused_result)) *xstrdup(const char *str)
         return ret;
 }
 
-static inline char * __attribute__((warn_unused_result)) xstrndup(const char *str, size_t size)
+static inline char * __attribute__((warn_unused_result)) __ul_returns_nonnull
+xstrndup(const char *str, size_t size)
 {
         char *ret;
 
