@@ -52,16 +52,18 @@ UL_DEBUG_DEFINE_MASKNAMES(loopdev) = UL_DEBUG_EMPTY_MASKNAMES;
 #define LOOPDEV_DEBUG_CXT	(1 << 2)
 #define LOOPDEV_DEBUG_ITER	(1 << 3)
 #define LOOPDEV_DEBUG_SETUP	(1 << 4)
-#define SFDISKPROG_DEBUG_ALL	0xFFFF
 
 #define DBG(m, x)       __UL_DBG(loopdev, LOOPDEV_DEBUG_, m, x)
 #define ON_DBG(m, x)    __UL_DBG_CALL(loopdev, LOOPDEV_DEBUG_, m, x)
 
+#define UL_DEBUG_CURRENT_MASK	UL_DEBUG_MASK(loopdev)
+#include "debugobj.h"
+
 static void loopdev_init_debug(void)
 {
 	if (loopdev_debug_mask)
 		return;
-	__UL_INIT_DEBUG(loopdev, LOOPDEV_DEBUG_, 0, LOOPDEV_DEBUG);
+	__UL_INIT_DEBUG_FROM_ENV(loopdev, LOOPDEV_DEBUG_, 0, LOOPDEV_DEBUG);
 }
 
 /*
@@ -94,6 +96,7 @@ int loopcxt_set_device(struct loopdev_cxt *lc, const char *device)
 	}
 	lc->fd = -1;
 	lc->mode = 0;
+	lc->blocksize = 0;
 	lc->has_info = 0;
 	lc->info_failed = 0;
 	*lc->device = '\0';
@@ -113,18 +116,18 @@ int loopcxt_set_device(struct loopdev_cxt *lc, const char *device)
 			}
 			snprintf(lc->device, sizeof(lc->device), "%s%s",
 				dir, device);
-		} else {
-			strncpy(lc->device, device, sizeof(lc->device));
-			lc->device[sizeof(lc->device) - 1] = '\0';
-		}
+		} else
+			xstrncpy(lc->device, device, sizeof(lc->device));
+
 		DBG(CXT, ul_debugobj(lc, "%s name assigned", device));
 	}
 
-	sysfs_deinit(&lc->sysfs);
+	ul_unref_path(lc->sysfs);
+	lc->sysfs = NULL;
 	return 0;
 }
 
-int loopcxt_has_device(struct loopdev_cxt *lc)
+inline int loopcxt_has_device(struct loopdev_cxt *lc)
 {
 	return lc && *lc->device;
 }
@@ -144,7 +147,7 @@ int loopcxt_has_device(struct loopdev_cxt *lc)
  * Note about LOOPDEV_FL_{RDONLY,RDWR} flags. These flags are used for open(2)
  * syscall to open loop device. By default is the device open read-only.
  *
- * The expection is loopcxt_setup_device(), where the device is open read-write
+ * The exception is loopcxt_setup_device(), where the device is open read-write
  * if LO_FLAGS_READ_ONLY flags is not set (see loopcxt_set_flags()).
  *
  * Returns: <0 on error, 0 on success.
@@ -241,24 +244,24 @@ const char *loopcxt_get_device(struct loopdev_cxt *lc)
  *
  * Returns pointer to the sysfs context (see lib/sysfs.c)
  */
-struct sysfs_cxt *loopcxt_get_sysfs(struct loopdev_cxt *lc)
+static struct path_cxt *loopcxt_get_sysfs(struct loopdev_cxt *lc)
 {
 	if (!lc || !*lc->device || (lc->flags & LOOPDEV_FL_NOSYSFS))
 		return NULL;
 
-	if (!lc->sysfs.devno) {
-		dev_t devno = sysfs_devname_to_devno(lc->device, NULL);
+	if (!lc->sysfs) {
+		dev_t devno = sysfs_devname_to_devno(lc->device);
 		if (!devno) {
 			DBG(CXT, ul_debugobj(lc, "sysfs: failed devname to devno"));
 			return NULL;
 		}
-		if (sysfs_init(&lc->sysfs, devno, NULL)) {
+
+		lc->sysfs = ul_new_sysfs_path(devno, NULL, NULL);
+		if (!lc->sysfs)
 			DBG(CXT, ul_debugobj(lc, "sysfs: init failed"));
-			return NULL;
-		}
 	}
 
-	return &lc->sysfs;
+	return lc->sysfs;
 }
 
 /*
@@ -359,7 +362,7 @@ int loopcxt_deinit_iterator(struct loopdev_cxt *lc)
 
 /*
  * Same as loopcxt_set_device, but also checks if the device is
- * associeted with any file.
+ * associated with any file.
  *
  * Returns: <0 on error, 0 on success, 1 device does not match with
  *         LOOPITER_FL_{USED,FREE} flags.
@@ -399,8 +402,8 @@ static int loopiter_set_device(struct loopdev_cxt *lc, const char *device)
 
 static int cmpnum(const void *p1, const void *p2)
 {
-	return (((* (int *) p1) > (* (int *) p2)) -
-			((* (int *) p1) < (* (int *) p2)));
+	return (((* (const int *) p1) > (* (const int *) p2)) -
+			((* (const int *) p1) < (* (const int *) p2)));
 }
 
 /*
@@ -683,15 +686,15 @@ struct loop_info64 *loopcxt_get_info(struct loopdev_cxt *lc)
  */
 char *loopcxt_get_backing_file(struct loopdev_cxt *lc)
 {
-	struct sysfs_cxt *sysfs = loopcxt_get_sysfs(lc);
+	struct path_cxt *sysfs = loopcxt_get_sysfs(lc);
 	char *res = NULL;
 
 	if (sysfs)
 		/*
-		 * This is always preffered, the loop_info64
+		 * This is always preferred, the loop_info64
 		 * has too small buffer for the filename.
 		 */
-		res = sysfs_strdup(sysfs, "loop/backing_file");
+		ul_path_read_string(sysfs, &res, "loop/backing_file");
 
 	if (!res && loopcxt_ioctl_enabled(lc)) {
 		struct loop_info64 *lo = loopcxt_get_info(lc);
@@ -715,11 +718,11 @@ char *loopcxt_get_backing_file(struct loopdev_cxt *lc)
  */
 int loopcxt_get_offset(struct loopdev_cxt *lc, uint64_t *offset)
 {
-	struct sysfs_cxt *sysfs = loopcxt_get_sysfs(lc);
+	struct path_cxt *sysfs = loopcxt_get_sysfs(lc);
 	int rc = -EINVAL;
 
 	if (sysfs)
-		rc = sysfs_read_u64(sysfs, "loop/offset", offset);
+		rc = ul_path_read_u64(sysfs, offset, "loop/offset");
 
 	if (rc && loopcxt_ioctl_enabled(lc)) {
 		struct loop_info64 *lo = loopcxt_get_info(lc);
@@ -743,11 +746,11 @@ int loopcxt_get_offset(struct loopdev_cxt *lc, uint64_t *offset)
  */
 int loopcxt_get_blocksize(struct loopdev_cxt *lc, uint64_t *blocksize)
 {
-	struct sysfs_cxt *sysfs = loopcxt_get_sysfs(lc);
+	struct path_cxt *sysfs = loopcxt_get_sysfs(lc);
 	int rc = -EINVAL;
 
 	if (sysfs)
-		rc = sysfs_read_u64(sysfs, "queue/logical_block_size", blocksize);
+		rc = ul_path_read_u64(sysfs, blocksize, "queue/logical_block_size");
 
 	/* Fallback based on BLKSSZGET ioctl */
 	if (rc) {
@@ -775,11 +778,11 @@ int loopcxt_get_blocksize(struct loopdev_cxt *lc, uint64_t *blocksize)
  */
 int loopcxt_get_sizelimit(struct loopdev_cxt *lc, uint64_t *size)
 {
-	struct sysfs_cxt *sysfs = loopcxt_get_sysfs(lc);
+	struct path_cxt *sysfs = loopcxt_get_sysfs(lc);
 	int rc = -EINVAL;
 
 	if (sysfs)
-		rc = sysfs_read_u64(sysfs, "loop/sizelimit", size);
+		rc = ul_path_read_u64(sysfs, size, "loop/sizelimit");
 
 	if (rc && loopcxt_ioctl_enabled(lc)) {
 		struct loop_info64 *lo = loopcxt_get_info(lc);
@@ -924,12 +927,12 @@ int loopmod_supports_partscan(void)
  */
 int loopcxt_is_partscan(struct loopdev_cxt *lc)
 {
-	struct sysfs_cxt *sysfs = loopcxt_get_sysfs(lc);
+	struct path_cxt *sysfs = loopcxt_get_sysfs(lc);
 
 	if (sysfs) {
 		/* kernel >= 3.2 */
 		int fl;
-		if (sysfs_read_int(sysfs, "loop/partscan", &fl) == 0)
+		if (ul_path_read_s32(sysfs, &fl, "loop/partscan") == 0)
 			return fl;
 	}
 
@@ -944,11 +947,11 @@ int loopcxt_is_partscan(struct loopdev_cxt *lc)
  */
 int loopcxt_is_autoclear(struct loopdev_cxt *lc)
 {
-	struct sysfs_cxt *sysfs = loopcxt_get_sysfs(lc);
+	struct path_cxt *sysfs = loopcxt_get_sysfs(lc);
 
 	if (sysfs) {
 		int fl;
-		if (sysfs_read_int(sysfs, "loop/autoclear", &fl) == 0)
+		if (ul_path_read_s32(sysfs, &fl, "loop/autoclear") == 0)
 			return fl;
 	}
 
@@ -967,11 +970,11 @@ int loopcxt_is_autoclear(struct loopdev_cxt *lc)
  */
 int loopcxt_is_readonly(struct loopdev_cxt *lc)
 {
-	struct sysfs_cxt *sysfs = loopcxt_get_sysfs(lc);
+	struct path_cxt *sysfs = loopcxt_get_sysfs(lc);
 
 	if (sysfs) {
 		int fl;
-		if (sysfs_read_int(sysfs, "ro", &fl) == 0)
+		if (ul_path_read_s32(sysfs, &fl, "ro") == 0)
 			return fl;
 	}
 
@@ -990,11 +993,11 @@ int loopcxt_is_readonly(struct loopdev_cxt *lc)
  */
 int loopcxt_is_dio(struct loopdev_cxt *lc)
 {
-	struct sysfs_cxt *sysfs = loopcxt_get_sysfs(lc);
+	struct path_cxt *sysfs = loopcxt_get_sysfs(lc);
 
 	if (sysfs) {
 		int fl;
-		if (sysfs_read_int(sysfs, "loop/dio", &fl) == 0)
+		if (ul_path_read_s32(sysfs, &fl, "loop/dio") == 0)
 			return fl;
 	}
 	if (loopcxt_ioctl_enabled(lc)) {
@@ -1030,8 +1033,8 @@ int loopcxt_is_used(struct loopdev_cxt *lc,
 		    uint64_t sizelimit,
 		    int flags)
 {
-	ino_t ino;
-	dev_t dev;
+	ino_t ino = 0;
+	dev_t dev = 0;
 
 	if (!lc)
 		return 0;
@@ -1063,12 +1066,12 @@ int loopcxt_is_used(struct loopdev_cxt *lc,
 	return 0;
 found:
 	if (flags & LOOPDEV_FL_OFFSET) {
-		uint64_t off;
+		uint64_t off = 0;
 
 		int rc = loopcxt_get_offset(lc, &off) == 0 && off == offset;
 
 		if (rc && flags & LOOPDEV_FL_SIZELIMIT) {
-			uint64_t sz;
+			uint64_t sz = 0;
 
 			return loopcxt_get_sizelimit(lc, &sz) == 0 && sz == sizelimit;
 		} else
@@ -1103,6 +1106,22 @@ int loopcxt_set_sizelimit(struct loopdev_cxt *lc, uint64_t sizelimit)
 	return 0;
 }
 
+/*
+ * The blocksize will be used by loopcxt_set_device(). For already exiting
+ * devices use  loopcxt_ioctl_blocksize().
+ *
+ * The setting is removed by loopcxt_set_device() loopcxt_next()!
+ */
+int loopcxt_set_blocksize(struct loopdev_cxt *lc, uint64_t blocksize)
+{
+	if (!lc)
+		return -EINVAL;
+	lc->blocksize = blocksize;
+
+	DBG(CXT, ul_debugobj(lc, "set blocksize=%jd", blocksize));
+	return 0;
+}
+
 /*
  * @lc: context
  * @flags: kernel LO_FLAGS_{READ_ONLY,USE_AOPS,AUTOCLEAR} flags
@@ -1138,8 +1157,7 @@ int loopcxt_set_backing_file(struct loopdev_cxt *lc, const char *filename)
 	if (!lc->filename)
 		return -errno;
 
-	strncpy((char *)lc->info.lo_file_name, lc->filename, LO_NAME_SIZE);
-	lc->info.lo_file_name[LO_NAME_SIZE- 1] = '\0';
+	xstrncpy((char *)lc->info.lo_file_name, lc->filename, LO_NAME_SIZE);
 
 	DBG(CXT, ul_debugobj(lc, "set backing file=%s", lc->info.lo_file_name));
 	return 0;
@@ -1213,7 +1231,7 @@ static int loopcxt_check_size(struct loopdev_cxt *lc, int file_fd)
 				      "size mismatch (%ju/%ju)",
 				      size, expected_size));
 
-		if (loopcxt_set_capacity(lc)) {
+		if (loopcxt_ioctl_capacity(lc)) {
 			/* ioctl not available */
 			if (errno == ENOTTY || errno == EINVAL)
 				errno = ERANGE;
@@ -1255,7 +1273,7 @@ static int loopcxt_check_size(struct loopdev_cxt *lc, int file_fd)
  */
 int loopcxt_setup_device(struct loopdev_cxt *lc)
 {
-	int file_fd, dev_fd, mode = O_RDWR, rc = -1, cnt = 0;
+	int file_fd, dev_fd, mode = O_RDWR, rc = -1, cnt = 0, err, again;
 	int errsv = 0;
 
 	if (!lc || !*lc->device || !lc->filename)
@@ -1328,7 +1346,19 @@ int loopcxt_setup_device(struct loopdev_cxt *lc)
 
 	DBG(SETUP, ul_debugobj(lc, "LOOP_SET_FD: OK"));
 
-	if (ioctl(dev_fd, LOOP_SET_STATUS64, &lc->info)) {
+	if (lc->blocksize > 0
+	    && (rc = loopcxt_ioctl_blocksize(lc, lc->blocksize)) < 0) {
+		errsv = -rc;
+		goto err;
+	}
+
+	do {
+		err = ioctl(dev_fd, LOOP_SET_STATUS64, &lc->info);
+		again = err && errno == EAGAIN;
+		if (again)
+			xusleep(250000);
+	} while (again);
+	if (err) {
 		rc = -errno;
 		errsv = errno;
 		DBG(SETUP, ul_debugobj(lc, "LOOP_SET_STATUS64 failed: %m"));
@@ -1371,9 +1401,9 @@ err:
  *
  * Returns: <0 on error, 0 on success.
  */
-int loopcxt_set_status(struct loopdev_cxt *lc)
+int loopcxt_ioctl_status(struct loopdev_cxt *lc)
 {
-	int dev_fd, rc = -1;
+	int dev_fd, rc = -1, err, again;
 
 	errno = 0;
 	dev_fd = loopcxt_get_fd(lc);
@@ -1384,7 +1414,13 @@ int loopcxt_set_status(struct loopdev_cxt *lc)
 	}
 	DBG(SETUP, ul_debugobj(lc, "device open: OK"));
 
-	if (ioctl(dev_fd, LOOP_SET_STATUS64, &lc->info)) {
+	do {
+		err = ioctl(dev_fd, LOOP_SET_STATUS64, &lc->info);
+		again = err && errno == EAGAIN;
+		if (again)
+			xusleep(250000);
+	} while (again);
+	if (err) {
 		rc = -errno;
 		DBG(SETUP, ul_debugobj(lc, "LOOP_SET_STATUS64 failed: %m"));
 		return rc;
@@ -1394,7 +1430,7 @@ int loopcxt_set_status(struct loopdev_cxt *lc)
 	return 0;
 }
 
-int loopcxt_set_capacity(struct loopdev_cxt *lc)
+int loopcxt_ioctl_capacity(struct loopdev_cxt *lc)
 {
 	int fd = loopcxt_get_fd(lc);
 
@@ -1412,7 +1448,7 @@ int loopcxt_set_capacity(struct loopdev_cxt *lc)
 	return 0;
 }
 
-int loopcxt_set_dio(struct loopdev_cxt *lc, unsigned long use_dio)
+int loopcxt_ioctl_dio(struct loopdev_cxt *lc, unsigned long use_dio)
 {
 	int fd = loopcxt_get_fd(lc);
 
@@ -1434,7 +1470,7 @@ int loopcxt_set_dio(struct loopdev_cxt *lc, unsigned long use_dio)
  * Kernel uses "unsigned long" as ioctl arg, but we use u64 for all sizes to
  * keep loopdev internal API simple.
  */
-int loopcxt_set_blocksize(struct loopdev_cxt *lc, uint64_t blocksize)
+int loopcxt_ioctl_blocksize(struct loopdev_cxt *lc, uint64_t blocksize)
 {
 	int fd = loopcxt_get_fd(lc);
 
@@ -1750,7 +1786,7 @@ char *loopdev_find_by_backing_file(const char *filename, uint64_t offset, uint64
 
 /*
  * Returns number of loop devices associated with @file, if only one loop
- * device is associeted with the given @filename and @loopdev is not NULL then
+ * device is associated with the given @filename and @loopdev is not NULL then
  * @loopdev returns name of the device.
  */
 int loopdev_count_by_backing_file(const char *filename, char **loopdev)
