@@ -13,41 +13,41 @@
 #include "mangle.h"
 #include "c.h"
 
-#define isoctal(a) (((a) & ~7) == '0')
+#define isoctal(a)		(((a) & ~7) == '0')
+
+#define from_hex(c)		(isdigit(c) ? c - '0' : tolower(c) - 'a' + 10)
+
+#define is_unwanted_char(x)	(strchr(" \t\n\\", (unsigned int) x) != NULL)
 
-static unsigned char need_escaping[] = { ' ', '\t', '\n', '\\' };
 
 char *mangle(const char *s)
 {
 	char *ss, *sp;
-	size_t n;
 
 	if (!s)
 		return NULL;
 
-	n = strlen(s);
-	ss = sp = malloc(4*n+1);
+	ss = sp = malloc(4 * strlen(s) + 1);
 	if (!sp)
 		return NULL;
 	while(1) {
-		for (n = 0; n < sizeof(need_escaping); n++) {
-			if (*s == need_escaping[n]) {
-				*sp++ = '\\';
-				*sp++ = '0' + ((*s & 0300) >> 6);
-				*sp++ = '0' + ((*s & 070) >> 3);
-				*sp++ = '0' + (*s & 07);
-				goto next;
-			}
-		}
-		*sp++ = *s;
-		if (*s == 0)
+		if (!*s) {
+			*sp = '\0';
 			break;
-	next:
+		}
+		if (is_unwanted_char(*s)) {
+			*sp++ = '\\';
+			*sp++ = '0' + ((*s & 0300) >> 6);
+			*sp++ = '0' + ((*s & 070) >> 3);
+			*sp++ = '0' + (*s & 07);
+		} else
+			*sp++ = *s;
 		s++;
 	}
 	return ss;
 }
 
+
 void unmangle_to_buffer(const char *s, char *buf, size_t len)
 {
 	size_t sz = 0;
@@ -56,7 +56,7 @@ void unmangle_to_buffer(const char *s, char *buf, size_t len)
 		return;
 
 	while(*s && sz < len - 1) {
-		if (*s == '\\' && sz + 4 < len - 1 && isoctal(s[1]) &&
+		if (*s == '\\' && sz + 3 < len - 1 && isoctal(s[1]) &&
 		    isoctal(s[2]) && isoctal(s[3])) {
 
 			*buf++ = 64*(s[1] & 7) + 8*(s[2] & 7) + (s[3] & 7);
@@ -70,20 +70,44 @@ void unmangle_to_buffer(const char *s, char *buf, size_t len)
 	*buf = '\0';
 }
 
-static inline char *skip_nonspaces(const char *s)
+size_t unhexmangle_to_buffer(const char *s, char *buf, size_t len)
 {
-	while (*s && !(*s == ' ' || *s == '\t'))
+	size_t sz = 0;
+	const char *buf0 = buf;
+
+	if (!s)
+		return 0;
+
+	while(*s && sz < len - 1) {
+		if (*s == '\\' && sz + 3 < len - 1 && s[1] == 'x' &&
+		    isxdigit(s[2]) && isxdigit(s[3])) {
+
+			*buf++ = from_hex(s[2]) << 4 | from_hex(s[3]);
+			s += 4;
+			sz += 4;
+		} else {
+			*buf++ = *s++;
+			sz++;
+		}
+	}
+	*buf = '\0';
+	return buf - buf0 + 1;
+}
+
+static inline const char *skip_nonspaces(const char *s)
+{
+	while (s && *s && !(*s == ' ' || *s == '\t'))
 		s++;
-	return (char *) s;
+	return s;
 }
 
 /*
  * Returns mallocated buffer or NULL in case of error.
  */
-char *unmangle(const char *s, char **end)
+char *unmangle(const char *s, const char **end)
 {
 	char *buf;
-	char *e;
+	const char *e;
 	size_t sz;
 
 	if (!s)
@@ -105,18 +129,22 @@ char *unmangle(const char *s, char **end)
 	return buf;
 }
 
-#ifdef TEST_PROGRAM
+#ifdef TEST_PROGRAM_MANGLE
 #include <errno.h>
 int main(int argc, char *argv[])
 {
+	char *p = NULL;
 	if (argc < 3) {
-		fprintf(stderr, "usage: %s --mangle | --unmangle <string>\n",
+		fprintf(stderr, "usage: %s --mangle|unmangle <string>\n",
 						program_invocation_short_name);
 		return EXIT_FAILURE;
 	}
 
-	if (!strcmp(argv[1], "--mangle"))
-		printf("mangled: '%s'\n", mangle(argv[2]));
+	if (!strcmp(argv[1], "--mangle")) {
+		p = mangle(argv[2]);
+		printf("mangled: '%s'\n", p);
+		free(p);
+	}
 
 	else if (!strcmp(argv[1], "--unmangle")) {
 		char *x = unmangle(argv[2], NULL);
@@ -127,9 +155,9 @@ int main(int argc, char *argv[])
 		}
 
 		x = strdup(argv[2]);
-		unmangle_to_buffer(x, x, strlen(x) + 1);
-
 		if (x) {
+			unmangle_to_buffer(x, x, strlen(x) + 1);
+
 			printf("self-unmangled: '%s'\n", x);
 			free(x);
 		}
@@ -137,4 +165,4 @@ int main(int argc, char *argv[])
 
 	return EXIT_SUCCESS;
 }
-#endif /* TEST_PROGRAM */
+#endif /* TEST_PROGRAM_MANGLE */
