@@ -52,12 +52,8 @@ int gettime_monotonic(struct timeval *tv)
 	int ret;
 	struct timespec ts;
 
-# ifdef CLOCK_MONOTONIC_RAW
 	/* Linux specific, can't slew */
-	if (!(ret = clock_gettime(CLOCK_MONOTONIC_RAW, &ts))) {
-# else
-	if (!(ret = clock_gettime(CLOCK_MONOTONIC, &ts))) {
-# endif
+	if (!(ret = clock_gettime(UL_CLOCK_MONOTONIC, &ts))) {
 		tv->tv_sec = ts.tv_sec;
 		tv->tv_usec = ts.tv_nsec / 1000;
 	}
