@@ -82,7 +82,7 @@ static int start_command(struct child_process *cmd)
 
 		cmd->preexec_cb();
 		execvp(cmd->argv[0], (char *const*) cmd->argv);
-		exit(127); /* cmd not found */
+		errexec(cmd->argv[0]);
 	}
 
 	if (cmd->pid < 0) {
@@ -141,11 +141,13 @@ static void pager_preexec(void)
 	 * Work around bug in "less" by not starting it until we
 	 * have real input
 	 */
-	fd_set in;
+	fd_set in, ex;
 
 	FD_ZERO(&in);
 	FD_SET(STDIN_FILENO, &in);
-	select(1, &in, NULL, &in, NULL);
+	ex = in;
+
+	select(STDIN_FILENO + 1, &in, NULL, &ex, NULL);
 
 	if (setenv("LESS", "FRSX", 0) != 0)
 		warn(_("failed to set the %s environment variable"), "LESS");
