@@ -75,10 +75,8 @@ struct file_attribute {
 /* Windows 10 Creators edition has extended the cluster size limit to 2MB */
 #define NTFS_MAX_CLUSTER_SIZE	(2 * 1024 * 1024)
 
-enum {
-	MFT_RECORD_ATTR_VOLUME_NAME		= 0x60,
-	MFT_RECORD_ATTR_END			= 0xffffffff
-};
+#define	MFT_RECORD_ATTR_VOLUME_NAME	0x60
+#define	MFT_RECORD_ATTR_END		0xffffffff
 
 static int probe_ntfs(blkid_probe pr, const struct blkid_idmag *mag)
 {
@@ -190,9 +188,9 @@ static int probe_ntfs(blkid_probe pr, const struct blkid_idmag *mag)
 		if (!attr_len)
 			break;
 
-		if (le32_to_cpu(attr->type) == MFT_RECORD_ATTR_END)
+		if (le32_to_cpu(attr->type) == (uint32_t) MFT_RECORD_ATTR_END)
 			break;
-		if (le32_to_cpu(attr->type) == MFT_RECORD_ATTR_VOLUME_NAME) {
+		if (le32_to_cpu(attr->type) == (uint32_t) MFT_RECORD_ATTR_VOLUME_NAME) {
 			unsigned int val_off = le16_to_cpu(attr->value_offset);
 			unsigned int val_len = le32_to_cpu(attr->value_len);
 			unsigned char *val = ((uint8_t *) attr) + val_off;
