@@ -130,8 +130,8 @@ void fdisk_unref_partition(struct fdisk_partition *pa)
 
 	pa->refcount--;
 	if (pa->refcount <= 0) {
-		fdisk_reset_partition(pa);
 		list_del(&pa->parts);
+		fdisk_reset_partition(pa);
 		DBG(PART, ul_debugobj(pa, "free"));
 		free(pa);
 	}
@@ -1019,7 +1019,9 @@ int fdisk_get_partition(struct fdisk_context *cxt, size_t partno,
 }
 
 static struct fdisk_partition *resize_get_by_offset(
-			struct fdisk_table *tb, fdisk_sector_t off)
+			struct fdisk_table *tb,
+			struct fdisk_partition *cur,
+			fdisk_sector_t off)
 {
 	struct fdisk_partition *pa = NULL;
 	struct fdisk_iter itr;
@@ -1029,6 +1031,9 @@ static struct fdisk_partition *resize_get_by_offset(
 	while (fdisk_table_next_partition(tb, &itr, &pa) == 0) {
 		if (!fdisk_partition_has_start(pa) || !fdisk_partition_has_size(pa))
 			continue;
+		if (fdisk_partition_is_nested(cur) &&
+		    pa->parent_partno != cur->parent_partno)
+			continue;
 		if (off >= pa->start && off < pa->start + pa->size)
 			return pa;
 	}
@@ -1113,7 +1118,7 @@ static int resize_get_last_possible(
 			break;
 		} else {
 			last = pa;
-			*maxsz += pa->size;
+			*maxsz = pa->size - (start - pa->start);
 			DBG(TAB, ul_debugobj(tb, "  new max=%ju (last updated)", (uintmax_t) *maxsz));
 		}
 	}
@@ -1147,8 +1152,10 @@ static int recount_resize(
 	rc = fdisk_get_partitions(cxt, &tb);
 	if (!rc)
 		rc = fdisk_get_freespaces(cxt, &tb);
-	if (rc)
+	if (rc) {
+		fdisk_unref_table(tb);
 		return rc;
+	}
 
 	fdisk_table_sort_partitions(tb, fdisk_partition_cmp_start);
 
@@ -1183,15 +1190,18 @@ static int recount_resize(
 
 	/* 2) verify that start is within the current partition or any freespace area */
 	if (!FDISK_IS_UNDEF(start)) {
-		struct fdisk_partition *area = resize_get_by_offset(tb, start);
+		struct fdisk_partition *area = resize_get_by_offset(tb, cur, start);
+
 		if (area == cur)
 			DBG(PART, ul_debugobj(tpl, "resize: start points to the current partition"));
 		else if (area && fdisk_partition_is_freespace(area))
 			DBG(PART, ul_debugobj(tpl, "resize: start points to freespace"));
 		else if (!area && start >= cxt->first_lba && start < cxt->first_lba + (cxt->grain / cxt->sector_size))
 			DBG(PART, ul_debugobj(tpl, "resize: start points before first partition"));
-		else
+		else {
+			DBG(PART, ul_debugobj(tpl, "resize: start verification failed"));
 			goto erange;
+		}
 	} else {
 		/* no change, start points to the current partition */
 		DBG(PART, ul_debugobj(tpl, "resize: start unchanged"));
