@@ -1,8 +1,13 @@
+/* SPDX-License-Identifier: LGPL-2.1-or-later */
 /*
- * Copyright (C) 2010 Karel Zak <kzak@redhat.com>
+ * This file is part of libmount from util-linux project.
  *
- * This file may be redistributed under the terms of the
- * GNU Lesser General Public License.
+ * Copyright (C) 2010-2018 Karel Zak <kzak@redhat.com>
+ *
+ * libmount is free software; you can redistribute it and/or modify it
+ * under the terms of the GNU Lesser General Public License as published by
+ * the Free Software Foundation; either version 2.1 of the License, or
+ * (at your option) any later version.
  */
 
 /**
@@ -51,6 +56,7 @@ int mnt_context_find_umount_fs(struct libmnt_context *cxt,
 			       struct libmnt_fs **pfs)
 {
 	int rc;
+	struct libmnt_ns *ns_old;
 	struct libmnt_table *mtab = NULL;
 	struct libmnt_fs *fs;
 	char *loopdev = NULL;
@@ -100,6 +106,10 @@ int mnt_context_find_umount_fs(struct libmnt_context *cxt,
 		return 1;
 	}
 
+	ns_old = mnt_context_switch_target_ns(cxt);
+	if (!ns_old)
+		return -MNT_ERR_NAMESPACE;
+
 try_loopdev:
 	fs = mnt_table_find_target(mtab, tgt, MNT_ITER_BACKWARD);
 	if (!fs && mnt_context_is_swapmatch(cxt)) {
@@ -159,12 +169,16 @@ try_loopdev:
 	if (pfs)
 		*pfs = fs;
 	free(loopdev);
+	if (!mnt_context_switch_ns(cxt, ns_old))
+		return -MNT_ERR_NAMESPACE;
 
 	DBG(CXT, ul_debugobj(cxt, "umount fs: %s", fs ? mnt_fs_get_target(fs) :
 							"<not found>"));
 	return fs ? 0 : 1;
 err:
 	free(loopdev);
+	if (!mnt_context_switch_ns(cxt, ns_old))
+		return -MNT_ERR_NAMESPACE;
 	return rc;
 }
 
@@ -317,8 +331,8 @@ static int lookup_umount_fs(struct libmnt_context *cxt)
 static int is_associated_fs(const char *devname, struct libmnt_fs *fs)
 {
 	uintmax_t offset = 0;
-	const char *src;
-	char *val, *optstr;
+	const char *src, *optstr;
+	char *val;
 	size_t valsz;
 	int flags = 0;
 
@@ -331,7 +345,7 @@ static int is_associated_fs(const char *devname, struct libmnt_fs *fs)
 		return 0;
 
 	/* check for the offset option in @fs */
-	optstr = (char *) mnt_fs_get_user_options(fs);
+	optstr = mnt_fs_get_user_options(fs);
 
 	if (optstr &&
 	    mnt_optstr_get_option(optstr, "offset", &val, &valsz) == 0) {
@@ -374,6 +388,56 @@ static int prepare_helper_from_options(struct libmnt_context *cxt,
 	return rc;
 }
 
+static int is_fuse_usermount(struct libmnt_context *cxt, int *errsv)
+{
+	struct libmnt_ns *ns_old;
+	const char *type = mnt_fs_get_fstype(cxt->fs);
+	const char *optstr;
+	char *user_id = NULL;
+	size_t sz;
+	uid_t uid;
+	char uidstr[sizeof(stringify_value(ULONG_MAX))];
+
+	*errsv = 0;
+
+	if (!type)
+		return 0;
+
+	if (strcmp(type, "fuse") != 0 &&
+	    strcmp(type, "fuseblk") != 0 &&
+	    strncmp(type, "fuse.", 5) != 0 &&
+	    strncmp(type, "fuseblk.", 8) != 0)
+		return 0;
+
+	/* get user_id= from mount table */
+	optstr = mnt_fs_get_fs_options(cxt->fs);
+	if (!optstr)
+		return 0;
+
+	if (mnt_optstr_get_option(optstr, "user_id", &user_id, &sz) != 0)
+		return 0;
+
+	if (sz == 0 || user_id == NULL)
+		return 0;
+
+	/* get current user */
+	ns_old = mnt_context_switch_origin_ns(cxt);
+	if (!ns_old) {
+		*errsv = -MNT_ERR_NAMESPACE;
+		return 0;
+	}
+
+	uid = getuid();
+
+	if (!mnt_context_switch_ns(cxt, ns_old)) {
+		*errsv = -MNT_ERR_NAMESPACE;
+		return 0;
+	}
+
+	snprintf(uidstr, sizeof(uidstr), "%lu", (unsigned long) uid);
+	return strncmp(user_id, uidstr, sz) == 0;
+}
+
 /*
  * Note that cxt->fs contains relevant mtab entry!
  */
@@ -382,7 +446,7 @@ static int evaluate_permissions(struct libmnt_context *cxt)
 	struct libmnt_table *fstab;
 	unsigned long u_flags = 0;
 	const char *tgt, *src, *optstr;
-	int rc, ok = 0;
+	int rc = 0, ok = 0;
 	struct libmnt_fs *fs;
 
 	assert(cxt);
@@ -410,6 +474,17 @@ static int evaluate_permissions(struct libmnt_context *cxt)
 			return 0;	/* we'll call /sbin/umount.<uhelper> */
 	}
 
+	/*
+	 * Check if this is a fuse mount for the current user,
+	 * if so then unmounting is allowed
+	 */
+	if (is_fuse_usermount(cxt, &rc)) {
+		DBG(CXT, ul_debugobj(cxt, "fuse user mount, umount is allowed"));
+		return 0;
+	}
+	if (rc)
+		return rc;
+
 	/*
 	 * User mounts have to be in /etc/fstab
 	 */
@@ -488,12 +563,21 @@ static int evaluate_permissions(struct libmnt_context *cxt)
 		char *curr_user;
 		char *mtab_user = NULL;
 		size_t sz;
+		struct libmnt_ns *ns_old;
 
 		DBG(CXT, ul_debugobj(cxt,
 				"umount: checking user=<username> from mtab"));
 
+		ns_old = mnt_context_switch_origin_ns(cxt);
+		if (!ns_old)
+			return -MNT_ERR_NAMESPACE;
+
 		curr_user = mnt_get_username(getuid());
 
+		if (!mnt_context_switch_ns(cxt, ns_old)) {
+			free(curr_user);
+			return -MNT_ERR_NAMESPACE;
+		}
 		if (!curr_user) {
 			DBG(CXT, ul_debugobj(cxt, "umount %s: cannot "
 				"convert %d to username", tgt, getuid()));
@@ -520,7 +604,10 @@ eperm:
 
 static int exec_helper(struct libmnt_context *cxt)
 {
+	char *namespace = NULL;
+	struct libmnt_ns *ns_tgt = mnt_context_get_target_ns(cxt);
 	int rc;
+	pid_t pid;
 
 	assert(cxt);
 	assert(cxt->fs);
@@ -534,12 +621,19 @@ static int exec_helper(struct libmnt_context *cxt)
 		return rc;
 	}
 
+	if (ns_tgt->fd != -1
+	    && asprintf(&namespace, "/proc/%i/fd/%i",
+			getpid(), ns_tgt->fd) == -1) {
+		return -ENOMEM;
+	}
+
 	DBG_FLUSH;
 
-	switch (fork()) {
+	pid = fork();
+	switch (pid) {
 	case 0:
 	{
-		const char *args[10], *type;
+		const char *args[12], *type;
 		int i = 0;
 
 		if (setgid(getgid()) < 0)
@@ -548,6 +642,9 @@ static int exec_helper(struct libmnt_context *cxt)
 		if (setuid(getuid()) < 0)
 			_exit(EXIT_FAILURE);
 
+		if (!mnt_context_switch_origin_ns(cxt))
+			_exit(EXIT_FAILURE);
+
 		type = mnt_fs_get_fstype(cxt->fs);
 
 		args[i++] = cxt->helper;			/* 1 */
@@ -567,10 +664,14 @@ static int exec_helper(struct libmnt_context *cxt)
 		    && strchr(type, '.')
 		    && !endswith(cxt->helper, type)) {
 			args[i++] = "-t";			/* 8 */
-			args[i++] = (char *) type;		/* 9 */
+			args[i++] = type;			/* 9 */
+		}
+		if (namespace) {
+			args[i++] = "-N";			/* 10 */
+			args[i++] = namespace;			/* 11 */
 		}
 
-		args[i] = NULL;					/* 10 */
+		args[i] = NULL;					/* 12 */
 		for (i = 0; args[i]; i++)
 			DBG(CXT, ul_debugobj(cxt, "argv[%d] = \"%s\"",
 							i, args[i]));
@@ -581,12 +682,18 @@ static int exec_helper(struct libmnt_context *cxt)
 	default:
 	{
 		int st;
-		wait(&st);
-		cxt->helper_status = WIFEXITED(st) ? WEXITSTATUS(st) : -1;
 
-		DBG(CXT, ul_debugobj(cxt, "%s executed [status=%d]",
-					cxt->helper, cxt->helper_status));
-		cxt->helper_exec_status = rc = 0;
+		if (waitpid(pid, &st, 0) == (pid_t) -1) {
+			cxt->helper_status = -1;
+			rc = -errno;
+		} else {
+			cxt->helper_status = WIFEXITED(st) ? WEXITSTATUS(st) : -1;
+			cxt->helper_exec_status = rc = 0;
+		}
+		DBG(CXT, ul_debugobj(cxt, "%s executed [status=%d, rc=%d%s]",
+				cxt->helper,
+				cxt->helper_status, rc,
+				rc ? " waitpid failed" : ""));
 		break;
 	}
 
@@ -635,6 +742,10 @@ int mnt_context_umount_setopt(struct libmnt_context *cxt, int c, char *arg)
 		if (arg)
 			rc = mnt_context_set_fstype(cxt, arg);
 		break;
+	case 'N':
+		if (arg)
+			rc = mnt_context_set_target_ns(cxt, arg);
+		break;
 	default:
 		return 1;
 	}
@@ -729,7 +840,7 @@ static int do_umount(struct libmnt_context *cxt)
 			-cxt->syscall_status));
 
 		rc = mount(src, mnt_fs_get_target(cxt->fs), NULL,
-			    MS_MGC_VAL | MS_REMOUNT | MS_RDONLY, NULL);
+			    MS_REMOUNT | MS_RDONLY, NULL);
 		if (rc < 0) {
 			cxt->syscall_status = -errno;
 			DBG(CXT, ul_debugobj(cxt,
@@ -765,6 +876,7 @@ static int do_umount(struct libmnt_context *cxt)
 int mnt_context_prepare_umount(struct libmnt_context *cxt)
 {
 	int rc;
+	struct libmnt_ns *ns_old;
 
 	if (!cxt || !cxt->fs || mnt_fs_is_swaparea(cxt->fs))
 		return -EINVAL;
@@ -780,6 +892,10 @@ int mnt_context_prepare_umount(struct libmnt_context *cxt)
 	cxt->helper = NULL;
 	cxt->action = MNT_ACT_UMOUNT;
 
+	ns_old = mnt_context_switch_target_ns(cxt);
+	if (!ns_old)
+		return -MNT_ERR_NAMESPACE;
+
 	rc = lookup_umount_fs(cxt);
 	if (!rc)
 		rc = mnt_context_merge_mflags(cxt);
@@ -813,6 +929,10 @@ int mnt_context_prepare_umount(struct libmnt_context *cxt)
 		return rc;
 	}
 	cxt->flags |= MNT_FL_PREPARED;
+
+	if (!mnt_context_switch_ns(cxt, ns_old))
+		return -MNT_ERR_NAMESPACE;
+
 	return rc;
 }
 
@@ -837,6 +957,7 @@ int mnt_context_prepare_umount(struct libmnt_context *cxt)
 int mnt_context_do_umount(struct libmnt_context *cxt)
 {
 	int rc;
+	struct libmnt_ns *ns_old;
 
 	assert(cxt);
 	assert(cxt->fs);
@@ -846,9 +967,13 @@ int mnt_context_do_umount(struct libmnt_context *cxt)
 	assert((cxt->action == MNT_ACT_UMOUNT));
 	assert((cxt->flags & MNT_FL_MOUNTFLAGS_MERGED));
 
+	ns_old = mnt_context_switch_target_ns(cxt);
+	if (!ns_old)
+		return -MNT_ERR_NAMESPACE;
+
 	rc = do_umount(cxt);
 	if (rc)
-		return rc;
+		goto end;
 
 	if (mnt_context_get_status(cxt) && !mnt_context_is_fake(cxt)) {
 		/*
@@ -873,6 +998,10 @@ int mnt_context_do_umount(struct libmnt_context *cxt)
 						       cxt->mountflags, NULL, cxt->fs);
 		}
 	}
+end:
+	if (!mnt_context_switch_ns(cxt, ns_old))
+		return -MNT_ERR_NAMESPACE;
+
 	return rc;
 }
 
@@ -927,6 +1056,7 @@ int mnt_context_finalize_umount(struct libmnt_context *cxt)
 int mnt_context_umount(struct libmnt_context *cxt)
 {
 	int rc;
+	struct libmnt_ns *ns_old;
 
 	assert(cxt);
 	assert(cxt->fs);
@@ -935,6 +1065,10 @@ int mnt_context_umount(struct libmnt_context *cxt)
 
 	DBG(CXT, ul_debugobj(cxt, "umount: %s", mnt_context_get_target(cxt)));
 
+	ns_old = mnt_context_switch_target_ns(cxt);
+	if (!ns_old)
+		return -MNT_ERR_NAMESPACE;
+
 	rc = mnt_context_prepare_umount(cxt);
 	if (!rc)
 		rc = mnt_context_prepare_update(cxt);
@@ -942,6 +1076,10 @@ int mnt_context_umount(struct libmnt_context *cxt)
 		rc = mnt_context_do_umount(cxt);
 	if (!rc)
 		rc = mnt_context_update_tabs(cxt);
+
+	if (!mnt_context_switch_ns(cxt, ns_old))
+		return -MNT_ERR_NAMESPACE;
+
 	return rc;
 }
 
@@ -995,11 +1133,12 @@ int mnt_context_next_umount(struct libmnt_context *cxt,
 	rc = mnt_context_get_mtab(cxt, &mtab);
 	cxt->mtab = NULL;		/* do not reset mtab */
 	mnt_reset_context(cxt);
-	cxt->mtab = mtab;
 
 	if (rc)
 		return rc;
 
+	cxt->mtab = mtab;
+
 	do {
 		rc = mnt_table_next_fs(mtab, itr, fs);
 		if (rc != 0)
@@ -1067,6 +1206,10 @@ int mnt_context_get_umount_excode(
 			if (buf)
 				snprintf(buf, bufsz, _("locking failed"));
 			return MNT_EX_FILEIO;
+		} else if (rc == -MNT_ERR_NAMESPACE) {
+			if (buf)
+				snprintf(buf, bufsz, _("failed to switch namespace"));
+			return MNT_EX_SYSERR;
 		}
 		return mnt_context_get_generic_excode(rc, buf, bufsz,
 					_("umount failed: %m"));
@@ -1080,6 +1223,10 @@ int mnt_context_get_umount_excode(
 			if (buf)
 				snprintf(buf, bufsz, _("filesystem was unmounted, but failed to update userspace mount table"));
 			return MNT_EX_FILEIO;
+		} else if (rc == -MNT_ERR_NAMESPACE) {
+			if (buf)
+				snprintf(buf, bufsz, _("filesystem was unmounted, but failed to switch namespace back"));
+			return MNT_EX_SYSERR;
 
 		} else if (rc < 0)
 			return mnt_context_get_generic_excode(rc, buf, bufsz,
