@@ -43,138 +43,201 @@ static void parser_cleanup(struct libmnt_parser *pa)
 	memset(pa, 0, sizeof(*pa));
 }
 
-static const char *next_number(const char *s, int *num, int *ok)
+static const char *next_s32(const char *s, int *num, int *rc)
 {
 	char *end = NULL;
 
-	assert(num);
-	assert(s);
-	assert(ok);
-
-	*ok = 0;
-	s = skip_blank(s);
 	if (!s || !*s)
 		return s;
 
+	*rc = -EINVAL;
 	*num = strtol(s, &end, 10);
 	if (end == NULL || s == end)
 	       return s;
-
-	/* valid end of number is a space or a terminator */
 	if (*end == ' ' || *end == '\t' || *end == '\0')
-		*ok = 1;
-
+		*rc = 0;
 	return end;
 }
 
+static const char *next_u64(const char *s, uint64_t *num, int *rc)
+{
+	char *end = NULL;
+
+	if (!s || !*s)
+		return s;
+
+	*rc = -EINVAL;
+	*num = (uint64_t) strtoumax(s, &end, 10);
+	if (end == NULL || s == end)
+	       return s;
+	if (*end == ' ' || *end == '\t' || *end == '\0')
+		*rc = 0;
+	return end;
+}
+
+static inline const char *skip_separator(const char *p)
+{
+	while (p && (*p == ' ' || *p == '\t'))
+		++p;
+	return p;
+}
+
+static inline const char *skip_nonspearator(const char *p)
+{
+	while (p && *p && !(*p == ' ' || *p == '\t'))
+		p++;
+	return p;
+}
+
 /*
  * Parses one line from {fs,m}tab
  */
 static int mnt_parse_table_line(struct libmnt_fs *fs, const char *s)
 {
-	int rc, n = 0, xrc;
-	char *src = NULL, *fstype = NULL, *optstr = NULL;
-
-	rc = sscanf(s,	UL_SCNsA" "	/* (1) source */
-			UL_SCNsA" "	/* (2) target */
-			UL_SCNsA" "	/* (3) FS type */
-			UL_SCNsA" "	/* (4) options */
-			"%n",		/* byte count */
-
-			&src,
-			&fs->target,
-			&fstype,
-			&optstr,
-			&n);
-	xrc = rc;
-
-	if (rc == 3 || rc == 4) {			/* options are optional */
-		unmangle_string(src);
-		unmangle_string(fs->target);
-		unmangle_string(fstype);
-
-		if (optstr && *optstr)
-			unmangle_string(optstr);
-
-		/* note that __foo functions do not reallocate the string
-		 */
-		rc = __mnt_fs_set_source_ptr(fs, src);
-		if (!rc) {
-			src = NULL;
-			rc = __mnt_fs_set_fstype_ptr(fs, fstype);
-			if (!rc)
-				fstype = NULL;
-		}
-		if (!rc && optstr)
-			rc = mnt_fs_set_options(fs, optstr);
-		free(optstr);
-		optstr = NULL;
-	} else {
-		DBG(TAB, ul_debug("tab parse error: [sscanf rc=%d]: '%s'", rc, s));
-		rc = -EINVAL;
-	}
-
-	if (rc) {
-		free(src);
-		free(fstype);
-		free(optstr);
-		DBG(TAB, ul_debug("tab parse error: [set vars, rc=%d]\n", rc));
-		return rc;	/* error */
-	}
+	int rc = 0;
+	char *p = NULL;
 
 	fs->passno = fs->freq = 0;
 
-	if (xrc == 4 && n)
-		s = skip_blank(s + n);
-	if (xrc == 4 && *s) {
-		int ok = 0;
-
-		s = next_number(s, &fs->freq, &ok);
-		if (!ok) {
-			if (s && *s) {
-				DBG(TAB, ul_debug("tab parse error: [freq]"));
-				rc = -EINVAL;
-			}
-		} else {
-			s = next_number(s, &fs->passno, &ok);
-			if (!ok && s && *s) {
-				DBG(TAB, ul_debug("tab parse error: [passno]"));
-				rc = -EINVAL;
-			}
-		}
+	/* (1) source */
+	p = unmangle(s, &s);
+	if (!p || (rc = __mnt_fs_set_source_ptr(fs, p))) {
+		DBG(TAB, ul_debug("tab parse error: [source]"));
+		free(p);
+		goto fail;
 	}
 
+	s = skip_separator(s);
+
+	/* (2) target */
+	fs->target = unmangle(s, &s);
+	if (!fs->target) {
+		DBG(TAB, ul_debug("tab parse error: [target]"));
+		goto fail;
+	}
+
+	s = skip_separator(s);
+
+	/* (3) FS type */
+	p = unmangle(s, &s);
+	if (!p || (rc = __mnt_fs_set_fstype_ptr(fs, p))) {
+		DBG(TAB, ul_debug("tab parse error: [fstype]"));
+		free(p);
+		goto fail;
+	}
+
+	s = skip_separator(s);
+
+	/* (4) options (optional) */
+	p = unmangle(s, &s);
+	if (p && (rc = mnt_fs_set_options(fs, p))) {
+		DBG(TAB, ul_debug("tab parse error: [options]"));
+		free(p);
+		goto fail;
+	}
+	if (!p)
+		goto done;
+	free(p);
+
+	s = skip_separator(s);
+	if (!s || !*s)
+		goto done;
+
+	/* (5) freq (optional) */
+	s = next_s32(s, &fs->freq, &rc);
+	if (s && *s && rc) {
+		DBG(TAB, ul_debug("tab parse error: [freq]"));
+		goto fail;
+	}
+
+	s = skip_separator(s);
+	if (!s || !*s)
+		goto done;
+
+	/* (6) freq (optional) */
+	s = next_s32(s, &fs->passno, &rc);
+	if (s && *s && rc) {
+		DBG(TAB, ul_debug("tab parse error: [passno]"));
+		goto fail;
+	}
+
+done:
+	return 0;
+fail:
+	if (rc == 0)
+		rc = -EINVAL;
+	DBG(TAB, ul_debug("tab parse error on: '%s' [rc=%d]", s, rc));
 	return rc;
 }
 
+
 /*
  * Parses one line from a mountinfo file
  */
 static int mnt_parse_mountinfo_line(struct libmnt_fs *fs, const char *s)
 {
-	int rc, end = 0;
+	int rc = 0;
 	unsigned int maj, min;
-	char *fstype = NULL, *src = NULL;
-	const char *p;
+	char *p;
 
-	rc = sscanf(s,	"%d "		/* (1) id */
-			"%d "		/* (2) parent */
-			"%u:%u "	/* (3) maj:min */
-			UL_SCNsA" "	/* (4) mountroot */
-			UL_SCNsA" "	/* (5) target */
-			UL_SCNsA	/* (6) vfs options (fs-independent) */
-			"%n",		/* number of read bytes */
+	fs->flags |= MNT_FS_KERNEL;
 
-			&fs->id,
-			&fs->parent,
-			&maj, &min,
-			&fs->root,
-			&fs->target,
-			&fs->vfs_optstr,
-			&end);
+	/* (1) id */
+	s = next_s32(s, &fs->id, &rc);
+	if (!s || !*s || rc) {
+		DBG(TAB, ul_debug("tab parse error: [id]"));
+		goto fail;
+	}
 
-	if (rc >= 7 && end > 0)
-		s += end;
+	s = skip_separator(s);
+
+	/* (2) parent */
+	s = next_s32(s, &fs->parent, &rc);
+	if (!s || !*s || rc) {
+		DBG(TAB, ul_debug("tab parse error: [parent]"));
+		goto fail;
+	}
+
+	s = skip_separator(s);
+
+	/* (3) maj:min */
+	if (sscanf(s, "%u:%u", &maj, &min) != 2) {
+		DBG(TAB, ul_debug("tab parse error: [maj:min]"));
+		goto fail;
+	}
+	fs->devno = makedev(maj, min);
+	s = skip_nonspearator(s);
+	s = skip_separator(s);
+
+	/* (4) mountroot */
+	fs->root = unmangle(s, &s);
+	if (!fs->root) {
+		DBG(TAB, ul_debug("tab parse error: [mountroot]"));
+		goto fail;
+	}
+
+	s = skip_separator(s);
+
+	/* (5) target */
+	fs->target = unmangle(s, &s);
+	if (!fs->target) {
+		DBG(TAB, ul_debug("tab parse error: [target]"));
+		goto fail;
+	}
+
+	/* remove "\040(deleted)" suffix */
+	p = (char *) endswith(fs->target, PATH_DELETED_SUFFIX);
+	if (p && *p)
+		*p = '\0';
+
+	s = skip_separator(s);
+
+	/* (6) vfs options (fs-independent) */
+	fs->vfs_optstr = unmangle(s, &s);
+	if (!fs->vfs_optstr) {
+		DBG(TAB, ul_debug("tab parse error: [VFS options]"));
+		goto fail;
+	}
 
 	/* (7) optional fields, terminated by " - " */
 	p = strstr(s, " - ");
@@ -184,82 +247,58 @@ static int mnt_parse_mountinfo_line(struct libmnt_fs *fs, const char *s)
 	}
 	if (p > s + 1)
 		fs->opt_fields = strndup(s + 1, p - s - 1);
-	s = p + 3;
 
-	end = 0;
-	rc += sscanf(s,	UL_SCNsA"%n",	/* (8) FS type */
+	s = skip_separator(p + 3);
 
-			&fstype,
-			&end);
-
-	if (rc >= 8 && end > 0)
-		s += end;
-	if (s[0] == ' ')
-		s++;
-
-	/* (9) source can unfortunately be an empty string "" and scanf does
-	 * not work well with empty string. Test with:
-	 *     $ sudo mount -t tmpfs "" /tmp/bb
-	 *     $ mountpoint /tmp/bb
-	 * */
-	if (s[0] == ' ') {
-		src = strdup("");
-		s++;
-		rc++;
-		rc += sscanf(s,	UL_SCNsA,	/* (10) fs options (fs specific) */
-
-				&fs->fs_optstr);
-	} else {
-		rc += sscanf(s,	UL_SCNsA" "	/* (9) source */
-				UL_SCNsA,	/* (10) fs options (fs specific) */
-
-				&src,
-				&fs->fs_optstr);
+	/* (8) FS type */
+	p = unmangle(s, &s);
+	if (!p || (rc = __mnt_fs_set_fstype_ptr(fs, p))) {
+		DBG(TAB, ul_debug("tab parse error: [fstype]"));
+		free(p);
+		goto fail;
 	}
 
-	if (rc >= 10) {
-		size_t sz;
-
-		fs->flags |= MNT_FS_KERNEL;
-		fs->devno = makedev(maj, min);
-
-		/* remove "\040(deleted)" suffix */
-		sz = strlen(fs->target);
-		if (sz > PATH_DELETED_SUFFIX_SZ) {
-			char *ptr = fs->target + (sz - PATH_DELETED_SUFFIX_SZ);
-
-			if (strcmp(ptr, PATH_DELETED_SUFFIX) == 0)
-				*ptr = '\0';
+	/* (9) source -- maybe empty string */
+	if (!s || !*s) {
+		DBG(TAB, ul_debug("tab parse error: [source]"));
+		goto fail;
+	} else if (*s == ' ' && *(s+1) == ' ') {
+		if ((rc = mnt_fs_set_source(fs, ""))) {
+			DBG(TAB, ul_debug("tab parse error: [empty source]"));
+			goto fail;
 		}
-
-		unmangle_string(fs->root);
-		unmangle_string(fs->target);
-		unmangle_string(fs->vfs_optstr);
-		unmangle_string(fstype);
-		unmangle_string(src);
-		unmangle_string(fs->fs_optstr);
-
-		rc = __mnt_fs_set_fstype_ptr(fs, fstype);
-		if (!rc) {
-			fstype = NULL;
-			rc = __mnt_fs_set_source_ptr(fs, src);
-			if (!rc)
-				src = NULL;
-		}
-
-		/* merge VFS and FS options to one string */
-		fs->optstr = mnt_fs_strdup_options(fs);
-		if (!fs->optstr)
-			rc = -ENOMEM;
 	} else {
-		DBG(TAB, ul_debug(
-			"mountinfo parse error [sscanf rc=%d]: '%s'", rc, s));
+		s = skip_separator(s);
+		p = unmangle(s, &s);
+		if (!p || (rc = __mnt_fs_set_source_ptr(fs, p))) {
+			DBG(TAB, ul_debug("tab parse error: [regular source]"));
+			free(p);
+			goto fail;
+		}
+	}
+
+	s = skip_separator(s);
+
+	/* (10) fs options (fs specific) */
+	fs->fs_optstr = unmangle(s, &s);
+	if (!fs->fs_optstr) {
+		DBG(TAB, ul_debug("tab parse error: [FS options]"));
+		goto fail;
+	}
+
+	/* merge VFS and FS options to one string */
+	fs->optstr = mnt_fs_strdup_options(fs);
+	if (!fs->optstr) {
+		rc = -ENOMEM;
+		DBG(TAB, ul_debug("tab parse error: [merge VFS and FS options]"));
+		goto fail;
+	}
+
+	return 0;
+fail:
+	if (rc == 0)
 		rc = -EINVAL;
-	}
-
-	free(fstype);
-	free(src);
-
+	DBG(TAB, ul_debug("tab parse error on: '%s' [rc=%d]", s, rc));
 	return rc;
 }
 
@@ -286,7 +325,8 @@ static int mnt_parse_utab_line(struct libmnt_fs *fs, const char *s)
 			char *v = unmangle(p + 4, &end);
 			if (!v)
 				goto enomem;
-			__mnt_fs_set_source_ptr(fs, v);
+			if (__mnt_fs_set_source_ptr(fs, v))
+				free(v);
 
 		} else if (!fs->target && !strncmp(p, "TARGET=", 7)) {
 			fs->target = unmangle(p + 7, &end);
@@ -332,48 +372,67 @@ enomem:
  */
 static int mnt_parse_swaps_line(struct libmnt_fs *fs, const char *s)
 {
-	uintmax_t fsz, usz;
-	int rc;
-	char *src = NULL;
+	uint64_t num;
+	int rc = 0;
+	char *p;
 
-	rc = sscanf(s,	UL_SCNsA" "	/* (1) source */
-			UL_SCNsA" "	/* (2) type */
-			"%ju"		/* (3) size */
-			"%ju"		/* (4) used */
-			"%d",		/* priority */
-
-			&src,
-			&fs->swaptype,
-			&fsz,
-			&usz,
-			&fs->priority);
-
-	if (rc == 5) {
-		size_t sz;
-
-		fs->size = fsz;
-		fs->usedsize = usz;
-
-		/* remove "\040(deleted)" suffix */
-		sz = strlen(src);
-		if (sz > PATH_DELETED_SUFFIX_SZ) {
-			char *p = src + (sz - PATH_DELETED_SUFFIX_SZ);
-			if (strcmp(p, PATH_DELETED_SUFFIX) == 0)
-				*p = '\0';
-		}
-
-		unmangle_string(src);
-
-		rc = mnt_fs_set_source(fs, src);
-		if (!rc)
-			mnt_fs_set_fstype(fs, "swap");
-	} else {
-		DBG(TAB, ul_debug("tab parse error: [sscanf rc=%d]: '%s'", rc, s));
-		rc = -EINVAL;
+	/* (1) source */
+	p = unmangle(s, &s);
+	if (p) {
+		char *x = (char *) endswith(p, PATH_DELETED_SUFFIX);
+		if (x && *x)
+			*x = '\0';
+	}
+	if (!p || (rc = __mnt_fs_set_source_ptr(fs, p))) {
+		DBG(TAB, ul_debug("tab parse error: [source]"));
+		free(p);
+		goto fail;
 	}
 
-	free(src);
+	s = skip_separator(s);
 
+	/* (2) type */
+	fs->swaptype = unmangle(s, &s);
+	if (!fs->swaptype) {
+		DBG(TAB, ul_debug("tab parse error: [swaptype]"));
+		goto fail;
+	}
+
+	s = skip_separator(s);
+
+	/* (3) size */
+	s = next_u64(s, &num, &rc);
+	if (!s || !*s || rc) {
+		DBG(TAB, ul_debug("tab parse error: [size]"));
+		goto fail;
+	}
+	fs->size = num;
+
+	s = skip_separator(s);
+
+	/* (4) size */
+	s = next_u64(s, &num, &rc);
+	if (!s || !*s || rc) {
+		DBG(TAB, ul_debug("tab parse error: [used size]"));
+		goto fail;
+	}
+	fs->usedsize = num;
+
+	s = skip_separator(s);
+
+	/* (5) priority */
+	s = next_s32(s, &fs->priority, &rc);
+	if (rc) {
+		DBG(TAB, ul_debug("tab parse error: [priority]"));
+		goto fail;
+	}
+
+	mnt_fs_set_fstype(fs, "swap");
+	return 0;
+fail:
+	if (rc == 0)
+		rc = -EINVAL;
+	DBG(TAB, ul_debug("tab parse error on: '%s' [rc=%d]", s, rc));
 	return rc;
 }
 
@@ -1027,9 +1086,10 @@ int mnt_table_parse_fstab(struct libmnt_table *tb, const char *filename)
 		return -EINVAL;
 	if (!filename)
 		filename = mnt_get_fstab_path();
-
-	if (!filename || stat(filename, &st))
+	if (!filename)
 		return -EINVAL;
+	if (stat(filename, &st) != 0)
+		return -errno;
 
 	tb->fmt = MNT_FMT_FSTAB;
 
