@@ -166,8 +166,7 @@ static void parse_argv(int argc, char **argv, struct sinfo *pinfo)
 	while ((c = getopt_long(argc, argv, "s:lhuv", long_options, NULL)) != -1) {
 		switch (c) {
 		case 'v':
-			printf(UTIL_LINUX_VERSION);
-			exit(EXIT_SUCCESS);
+			print_version(EXIT_SUCCESS);
 		case 'u': /* deprecated */
 		case 'h':
 			usage();
@@ -264,7 +263,7 @@ int main(int argc, char **argv)
 	setlocale(LC_ALL, "");
 	bindtextdomain(PACKAGE, LOCALEDIR);
 	textdomain(PACKAGE);
-	atexit(close_stdout);
+	close_stdout_atexit();
 
 	parse_argv(argc, argv, &info);
 	if (!info.username) {
