@@ -77,6 +77,7 @@
 #include "all-io.h"
 #include "fileutils.h"
 #include "ttyutils.h"
+#include "pwdutils.h"
 
 #include "logindefs.h"
 
@@ -104,9 +105,12 @@ struct login_context {
 	const char	*tty_number;	/* end of the tty_path */
 	mode_t		tty_mode;	/* chmod() mode */
 
-	char		*username;	/* from command line or PAM */
+	const char	*username;	/* points to PAM, pwd or cmd_username */
+	char            *cmd_username;	/* username specified on command line */
+
 
 	struct passwd	*pwd;		/* user info */
+	char		*pwdbuf;	/* pwd strings */
 
 	pam_handle_t	*pamh;		/* PAM handler */
 	struct pam_conv	conv;		/* PAM conversation */
@@ -137,6 +141,7 @@ struct login_context {
 static unsigned int timeout = LOGIN_TIMEOUT;
 static int child_pid = 0;
 static volatile int got_sig = 0;
+static char timeout_msg[128];
 
 #ifdef LOGIN_CHOWN_VCS
 /* true if the filedescriptor fd is a console tty, very Linux specific */
@@ -172,15 +177,14 @@ timedout2(int sig __attribute__ ((__unused__)))
 	tcgetattr(0, &ti);
 	ti.c_lflag |= ECHO;
 	tcsetattr(0, TCSANOW, &ti);
-	exit(EXIT_SUCCESS);	/* %% */
+	_exit(EXIT_SUCCESS);	/* %% */
 }
 
 static void timedout(int sig __attribute__ ((__unused__)))
 {
 	signal(SIGALRM, timedout2);
 	alarm(10);
-	/* TRANSLATORS: The standard value for %u is 60. */
-	warnx(_("timed out after %u seconds"), timeout);
+	ignore_result( write(STDERR_FILENO, timeout_msg, strlen(timeout_msg)) );
 	signal(SIGALRM, SIG_IGN);
 	alarm(0);
 	timedout2(0);
@@ -433,14 +437,14 @@ static void log_btmp(struct login_context *cxt)
 
 	memset(&ut, 0, sizeof(ut));
 
-	strncpy(ut.ut_user,
+	str2memcpy(ut.ut_user,
 		cxt->username ? cxt->username : "(unknown)",
 		sizeof(ut.ut_user));
 
 	if (cxt->tty_number)
-		strncpy(ut.ut_id, cxt->tty_number, sizeof(ut.ut_id));
+		str2memcpy(ut.ut_id, cxt->tty_number, sizeof(ut.ut_id));
 	if (cxt->tty_name)
-		xstrncpy(ut.ut_line, cxt->tty_name, sizeof(ut.ut_line));
+		str2memcpy(ut.ut_line, cxt->tty_name, sizeof(ut.ut_line));
 
 	gettimeofday(&tv, NULL);
 	ut.ut_tv.tv_sec = tv.tv_sec;
@@ -450,7 +454,7 @@ static void log_btmp(struct login_context *cxt)
 	ut.ut_pid = cxt->pid;
 
 	if (cxt->hostname) {
-		xstrncpy(ut.ut_host, cxt->hostname, sizeof(ut.ut_host));
+		str2memcpy(ut.ut_host, cxt->hostname, sizeof(ut.ut_host));
 		if (*cxt->hostaddress)
 			memcpy(&ut.ut_addr_v6, cxt->hostaddress,
 			       sizeof(ut.ut_addr_v6));
@@ -499,6 +503,9 @@ static void log_lastlog(struct login_context *cxt)
 	if (!cxt->pwd)
 		return;
 
+	if (cxt->pwd->pw_uid > (uid_t) getlogindefs_num("LASTLOG_UID_MAX", ULONG_MAX))
+		return;
+
 	/* lastlog is huge on systems with large UIDs, ignore SIGXFSZ */
 	memset(&sa, 0, sizeof(sa));
 	sa.sa_handler = SIG_IGN;
@@ -537,9 +544,9 @@ static void log_lastlog(struct login_context *cxt)
 	ll.ll_time = t;		/* ll_time is always 32bit */
 
 	if (cxt->tty_name)
-		xstrncpy(ll.ll_line, cxt->tty_name, sizeof(ll.ll_line));
+		str2memcpy(ll.ll_line, cxt->tty_name, sizeof(ll.ll_line));
 	if (cxt->hostname)
-		xstrncpy(ll.ll_host, cxt->hostname, sizeof(ll.ll_host));
+		str2memcpy(ll.ll_host, cxt->hostname, sizeof(ll.ll_host));
 
 	if (write_all(fd, (char *)&ll, sizeof(ll)))
 		warn(_("write lastlog failed"));
@@ -581,7 +588,7 @@ static void log_utmp(struct login_context *cxt)
 	if (utp == NULL && cxt->tty_name) {
 		setutxent();
 		ut.ut_type = LOGIN_PROCESS;
-		strncpy(ut.ut_line, cxt->tty_name, sizeof(ut.ut_line));
+		str2memcpy(ut.ut_line, cxt->tty_name, sizeof(ut.ut_line));
 		utp = getutxline(&ut);
 	}
 
@@ -590,7 +597,7 @@ static void log_utmp(struct login_context *cxt)
 	if (utp == NULL && cxt->tty_number) {
 	     setutxent();
 	     ut.ut_type = DEAD_PROCESS;
-	     strncpy(ut.ut_id, cxt->tty_number, sizeof(ut.ut_id));
+	     str2memcpy(ut.ut_id, cxt->tty_number, sizeof(ut.ut_id));
 	     utp = getutxid(&ut);
 	}
 
@@ -601,11 +608,11 @@ static void log_utmp(struct login_context *cxt)
 		memset(&ut, 0, sizeof(ut));
 
 	if (cxt->tty_number && ut.ut_id[0] == 0)
-		strncpy(ut.ut_id, cxt->tty_number, sizeof(ut.ut_id));
+		str2memcpy(ut.ut_id, cxt->tty_number, sizeof(ut.ut_id));
 	if (cxt->username)
-		strncpy(ut.ut_user, cxt->username, sizeof(ut.ut_user));
+		str2memcpy(ut.ut_user, cxt->username, sizeof(ut.ut_user));
 	if (cxt->tty_name)
-		xstrncpy(ut.ut_line, cxt->tty_name, sizeof(ut.ut_line));
+		str2memcpy(ut.ut_line, cxt->tty_name, sizeof(ut.ut_line));
 
 	gettimeofday(&tv, NULL);
 	ut.ut_tv.tv_sec = tv.tv_sec;
@@ -613,7 +620,7 @@ static void log_utmp(struct login_context *cxt)
 	ut.ut_type = USER_PROCESS;
 	ut.ut_pid = cxt->pid;
 	if (cxt->hostname) {
-		xstrncpy(ut.ut_host, cxt->hostname, sizeof(ut.ut_host));
+		str2memcpy(ut.ut_host, cxt->hostname, sizeof(ut.ut_host));
 		if (*cxt->hostaddress)
 			memcpy(&ut.ut_addr_v6, cxt->hostaddress,
 			       sizeof(ut.ut_addr_v6));
@@ -652,33 +659,13 @@ static void log_syslog(struct login_context *cxt)
 	}
 }
 
-static struct passwd *get_passwd_entry(const char *username,
-					 char **pwdbuf,
-					 struct passwd *pwd)
-{
-	struct passwd *res = NULL;
-	int x;
-
-	if (!pwdbuf || !username)
-		return NULL;
-
-	*pwdbuf = xrealloc(*pwdbuf, UL_GETPW_BUFSIZ);
-
-	x = getpwnam_r(username, pwd, *pwdbuf, UL_GETPW_BUFSIZ, &res);
-	if (!res) {
-		errno = x;
-		return NULL;
-	}
-	return res;
-}
-
 /* encapsulate stupid "void **" pam_get_item() API */
-static int loginpam_get_username(pam_handle_t *pamh, char **name)
+static int loginpam_get_username(pam_handle_t *pamh, const char **name)
 {
-	const void *item = (void *)*name;
+	const void *item = (const void *)*name;
 	int rc;
 	rc = pam_get_item(pamh, PAM_USER, &item);
-	*name = (char *)item;
+	*name = (const char *)item;
 	return rc;
 }
 
@@ -695,7 +682,8 @@ static void loginpam_err(pam_handle_t *pamh, int retcode)
 }
 
 /*
- * Composes "<host> login: " string; or returns "login: " if -H is given.
+ * Composes "<host> login: " string; or returns "login: " if -H is given or
+ * LOGIN_PLAIN_PROMPT=yes configured.
  */
 static const char *loginpam_get_prompt(struct login_context *cxt)
 {
@@ -703,11 +691,16 @@ static const char *loginpam_get_prompt(struct login_context *cxt)
 	char *prompt, *dflt_prompt = _("login: ");
 	size_t sz;
 
-	if (cxt->nohost || !(host = get_thishost(cxt, NULL)))
+	if (cxt->nohost)
+		return dflt_prompt;	/* -H on command line */
+
+	if (getlogindefs_bool("LOGIN_PLAIN_PROMPT", 0) == 1)
+		return dflt_prompt;
+
+	if (!(host = get_thishost(cxt, NULL)))
 		return dflt_prompt;
 
 	sz = strlen(host) + 1 + strlen(dflt_prompt) + 1;
-
 	prompt = xmalloc(sz);
 	snprintf(prompt, sz, "%s %s", host, dflt_prompt);
 
@@ -752,7 +745,6 @@ static pam_handle_t *init_loginpam(struct login_context *cxt)
 		loginpam_err(pamh, rc);
 
 	/* We don't need the original username. We have to follow PAM. */
-	free(cxt->username);
 	cxt->username = NULL;
 	cxt->pamh = pamh;
 
@@ -864,8 +856,7 @@ static void loginpam_acct(struct login_context *cxt)
 
 	if (!cxt->username || !*cxt->username) {
 		warnx(_("\nSession setup problem, abort."));
-		syslog(LOG_ERR, _("NULL user name in %s:%d. Abort."),
-		       __FUNCTION__, __LINE__);
+		syslog(LOG_ERR, _("NULL user name. Abort."));
 		pam_end(pamh, PAM_SYSTEM_ERR);
 		sleepexit(EXIT_FAILURE);
 	}
@@ -963,6 +954,8 @@ static void fork_session(struct login_context *cxt)
 		close(0);
 		close(1);
 		close(2);
+		free_getlogindefs_data();
+
 		sa.sa_handler = SIG_IGN;
 		sigaction(SIGQUIT, &sa, NULL);
 		sigaction(SIGINT, &sa, NULL);
@@ -1102,6 +1095,24 @@ static void init_remote_info(struct login_context *cxt, char *remotehost)
 	}
 }
 
+static void __attribute__((__noreturn__)) usage(void)
+{
+	fputs(USAGE_HEADER, stdout);
+	printf(_(" %s [-p] [-h <host>] [-H] [[-f] <username>]\n"), program_invocation_short_name);
+	fputs(USAGE_SEPARATOR, stdout);
+	fputs(_("Begin a session on the system.\n"), stdout);
+
+	fputs(USAGE_OPTIONS, stdout);
+	puts(_(" -p             do not destroy the environment"));
+	puts(_(" -f             skip a second login authentication"));
+	puts(_(" -h <host>      hostname to be used for utmp logging"));
+	puts(_(" -H             suppress hostname in the login prompt"));
+	printf("     --help     %s\n", USAGE_OPTSTR_HELP);
+	printf(" -V, --version  %s\n", USAGE_OPTSTR_VERSION);
+	printf(USAGE_MAN_TAIL("login(1)"));
+	exit(EXIT_SUCCESS);
+}
+
 int main(int argc, char **argv)
 {
 	int c;
@@ -1111,9 +1122,7 @@ int main(int argc, char **argv)
 	int childArgc = 0;
 	int retcode;
 	struct sigaction act;
-
-	char *pwdbuf = NULL;
-	struct passwd *pwd = NULL, _pwd;
+	struct passwd *pwd;
 
 	struct login_context cxt = {
 		.tty_mode = TTY_MODE,		  /* tty chmod() */
@@ -1126,8 +1135,25 @@ int main(int argc, char **argv)
 
 	};
 
+	/* the only two longopts to satisfy UL standards */
+	enum { HELP_OPTION = CHAR_MAX + 1 };
+	static const struct option longopts[] = {
+		{"help", no_argument, NULL, HELP_OPTION},
+		{"version", no_argument, NULL, 'V'},
+		{NULL, 0, NULL, 0}
+	};
+
 	timeout = (unsigned int)getlogindefs_num("LOGIN_TIMEOUT", LOGIN_TIMEOUT);
 
+	setlocale(LC_ALL, "");
+	bindtextdomain(PACKAGE, LOCALEDIR);
+	textdomain(PACKAGE);
+
+	/* TRANSLATORS: The standard value for %u is 60. */
+	snprintf(timeout_msg, sizeof(timeout_msg),
+	    _("%s: timed out after %u seconds"),
+	    program_invocation_short_name, timeout);
+
 	signal(SIGALRM, timedout);
 	(void) sigaction(SIGALRM, NULL, &act);
 	act.sa_flags &= ~SA_RESTART;
@@ -1136,10 +1162,6 @@ int main(int argc, char **argv)
 	signal(SIGQUIT, SIG_IGN);
 	signal(SIGINT, SIG_IGN);
 
-	setlocale(LC_ALL, "");
-	bindtextdomain(PACKAGE, LOCALEDIR);
-	textdomain(PACKAGE);
-
 	setpriority(PRIO_PROCESS, 0, 0);
 	initproctitle(argc, argv);
 
@@ -1149,7 +1171,7 @@ int main(int argc, char **argv)
 	 * -h is used by other servers to pass the name of the remote
 	 *    host to login so that it may be placed in utmp and wtmp
 	 */
-	while ((c = getopt(argc, argv, "fHh:pV")) != -1)
+	while ((c = getopt_long(argc, argv, "fHh:pV", longopts, NULL)) != -1)
 		switch (c) {
 		case 'f':
 			cxt.noauth = 1;
@@ -1173,21 +1195,22 @@ int main(int argc, char **argv)
 			break;
 
 		case 'V':
-			printf(UTIL_LINUX_VERSION);
-			return EXIT_SUCCESS;
-		case '?':
+			print_version(EXIT_SUCCESS);
+		case HELP_OPTION:
+			usage();
 		default:
-			fprintf(stderr, _("Usage: login [-p] [-h <host>] [-H] [[-f] <username>]\n"));
-			fputs(USAGE_SEPARATOR, stderr);
-			fputs(_("Begin a session on the system.\n"), stderr);
-			exit(EXIT_FAILURE);
+			errtryhelp(EXIT_FAILURE);
 		}
 	argc -= optind;
 	argv += optind;
 
 	if (*argv) {
 		char *p = *argv;
-		cxt.username = xstrdup(p);
+
+		/* username from command line */
+		cxt.cmd_username = xstrdup(p);
+		/* used temporary, it'll be replaced by username from PAM or/and cxt->pwd */
+		cxt.username = cxt.cmd_username;
 
 		/* Wipe the name - some people mistype their password here. */
 		/* (Of course we are too late, but perhaps this helps a little...) */
@@ -1219,10 +1242,11 @@ int main(int argc, char **argv)
 	 */
 	loginpam_acct(&cxt);
 
-	if (!(cxt.pwd = get_passwd_entry(cxt.username, &pwdbuf, &_pwd))) {
+	cxt.pwd = xgetpwnam(cxt.username, &cxt.pwdbuf);
+	if (!cxt.pwd) {
 		warnx(_("\nSession setup problem, abort."));
-		syslog(LOG_ERR, _("Invalid user name \"%s\" in %s:%d. Abort."),
-		       cxt.username, __FUNCTION__, __LINE__);
+		syslog(LOG_ERR, _("Invalid user name \"%s\". Abort."),
+		       cxt.username);
 		pam_end(cxt.pamh, PAM_SYSTEM_ERR);
 		sleepexit(EXIT_FAILURE);
 	}
