@@ -62,12 +62,17 @@ struct wipe_desc {
 };
 
 struct wipe_control {
-	const char	*devname;
+	char		*devname;
 	const char	*type_pattern;		/* -t <pattern> */
 
 	struct libscols_table *outtab;
 	struct wipe_desc *offsets;		/* -o <offset> -o <offset> ... */
 
+	size_t		ndevs;			/* number of devices to probe */
+
+	char		**reread;		/* devices to BLKRRPART */
+	size_t		nrereads;		/* size of reread */
+
 	unsigned int	noact : 1,
 			all : 1,
 			quiet : 1,
@@ -163,11 +168,19 @@ static void init_output(struct wipe_control *ctl)
 
 	for (i = 0; i < ncolumns; i++) {
 		const struct colinfo *col = get_column_info(i);
+		struct libscols_column *cl;
 
-		if (!scols_table_new_column(tb, col->name, col->whint,
-					    col->flags))
+		cl = scols_table_new_column(tb, col->name, col->whint,
+					    col->flags);
+		if (!cl)
 			err(EXIT_FAILURE,
 			    _("failed to initialize output column"));
+		if (ctl->json) {
+			int id = get_column_id(i);
+
+			if (id == COL_LEN)
+				scols_column_set_json_type(cl, SCOLS_JSON_NUMBER);
+		}
 	}
 	ctl->outtab = tb;
 }
@@ -294,7 +307,7 @@ static struct wipe_desc *get_desc_for_probe(struct wipe_control *ctl,
 					    loff_t *offset,
 					    size_t *len)
 {
-	const char *off, *type, *mag, *p, *usage = NULL;
+	const char *off, *type, *mag, *p, *use = NULL;
 	struct wipe_desc *wp;
 	int rc, ispt = 0;
 
@@ -315,7 +328,7 @@ static struct wipe_desc *get_desc_for_probe(struct wipe_control *ctl,
 			rc = blkid_probe_lookup_value(pr, "PTMAGIC", &mag, len);
 		if (rc)
 			return NULL;
-		usage = N_("partition-table");
+		use = N_("partition-table");
 		ispt = 1;
 	} else
 		return NULL;
@@ -344,8 +357,8 @@ static struct wipe_desc *get_desc_for_probe(struct wipe_control *ctl,
 	if (!wp)
 		return NULL;
 
-	if (usage || blkid_probe_lookup_value(pr, "USAGE", &usage, NULL) == 0)
-		wp->usage = xstrdup(usage);
+	if (use || blkid_probe_lookup_value(pr, "USAGE", &use, NULL) == 0)
+		wp->usage = xstrdup(use);
 
 	wp->type = xstrdup(type);
 	wp->on_disk = 1;
@@ -496,12 +509,25 @@ err:
 static void rereadpt(int fd, const char *devname)
 {
 	struct stat st;
+	int try = 0;
 
 	if (fstat(fd, &st) || !S_ISBLK(st.st_mode))
 		return;
 
-	errno = 0;
-	ioctl(fd, BLKRRPART);
+	do {
+		/*
+		 * Unfortunately, it's pretty common that the first re-read
+		 * without delay is uncuccesful. The reason is probably kernel
+		 * and/or udevd.  Let's wait a moment and try more attempts.
+		 */
+		xusleep(25000);
+
+		errno = 0;
+		ioctl(fd, BLKRRPART);
+		if (errno != EBUSY)
+			break;
+	} while (try++ < 4);
+
 	printf(_("%s: calling ioctl to re-read partition table: %m\n"), devname);
 }
 #endif
@@ -582,8 +608,21 @@ static int do_wipe(struct wipe_control *ctl)
 	fsync(blkid_probe_get_fd(pr));
 
 #ifdef BLKRRPART
-	if (reread && (mode & O_EXCL))
-		rereadpt(blkid_probe_get_fd(pr), ctl->devname);
+	if (reread && (mode & O_EXCL)) {
+		if (ctl->ndevs > 1) {
+			/*
+			 * We're going to probe more device, let's postpone
+			 * re-read PT ioctl until all is erased to avoid
+			 * situation we erase PT on /dev/sda before /dev/sdaN
+			 * devices are processed.
+			 */
+			if (!ctl->reread)
+				ctl->reread = xcalloc(ctl->ndevs, sizeof(char *));
+
+			ctl->reread[ctl->nrereads++] = ctl->devname;
+		} else
+			rereadpt(blkid_probe_get_fd(pr), ctl->devname);
+	}
 #endif
 
 	close(blkid_probe_get_fd(pr));
@@ -661,7 +700,7 @@ main(int argc, char **argv)
 	setlocale(LC_ALL, "");
 	bindtextdomain(PACKAGE, LOCALEDIR);
 	textdomain(PACKAGE);
-	atexit(close_stdout);
+	close_stdout_atexit();
 
 	while ((c = getopt_long(argc, argv, "abfhiJnO:o:pqt:V", longopts, NULL)) != -1) {
 
@@ -677,9 +716,6 @@ main(int argc, char **argv)
 		case 'f':
 			ctl.force = 1;
 			break;
-		case 'h':
-			usage();
-			break;
 		case 'J':
 			ctl.json = 1;
 			break;
@@ -706,9 +742,11 @@ main(int argc, char **argv)
 		case 't':
 			ctl.type_pattern = optarg;
 			break;
+
+		case 'h':
+			usage();
 		case 'V':
-			printf(UTIL_LINUX_VERSION);
-			return EXIT_SUCCESS;
+			print_version(EXIT_SUCCESS);
 		default:
 			errtryhelp(EXIT_FAILURE);
 		}
@@ -763,11 +801,29 @@ main(int argc, char **argv)
 		/*
 		 * Erase
 		 */
+		ctl.ndevs = argc - optind;
+
 		while (optind < argc) {
 			ctl.devname = argv[optind++];
 			do_wipe(&ctl);
+			ctl.ndevs--;
 		}
-	}
 
+#ifdef BLKRRPART
+		/* Re-read partition tables on whole-disk devices. This is
+		 * postponed until all is done to avoid conflicts.
+		 */
+		for (size_t i = 0; i < ctl.nrereads; i++) {
+			char *devname = ctl.reread[i];
+			int fd = open(devname, O_RDONLY);
+
+			if (fd >= 0) {
+				rereadpt(fd, devname);
+				close(fd);
+			}
+		}
+		free(ctl.reread);
+#endif
+	}
 	return EXIT_SUCCESS;
 }
