@@ -1017,6 +1017,8 @@ full_output:
 		if (!line) {
 			mesg_copy = xstrdup(rec->mesg);
 			line = strtok(mesg_copy, "\n");
+			if (!line)
+				goto done;	/* only when something is wrong */
 			mesg_size = strlen(line);
 		}
 	} else {
@@ -1057,6 +1059,7 @@ full_output:
 		free(mesg_copy);
 	}
 
+done:
 	putchar('\n');
 }
 
@@ -1354,7 +1357,7 @@ int main(int argc, char *argv[])
 	setlocale(LC_ALL, "");
 	bindtextdomain(PACKAGE, LOCALEDIR);
 	textdomain(PACKAGE);
-	atexit(close_stdout);
+	close_stdout_atexit();
 
 	while ((c = getopt_long(argc, argv, "CcDdEeF:f:HhkL::l:n:iPprSs:TtuVwx",
 				longopts, NULL)) != -1) {
@@ -1395,9 +1398,6 @@ int main(int argc, char *argv[])
 			colormode = UL_COLORMODE_AUTO;
 			ctl.pager = 1;
 			break;
-		case 'h':
-			usage();
-			break;
 		case 'k':
 			ctl.fltr_fac = 1;
 			setbit(ctl.facilities, FAC_BASE(LOG_KERN));
@@ -1447,9 +1447,6 @@ int main(int argc, char *argv[])
 			for (n = 1; (size_t) n < ARRAY_SIZE(facility_names); n++)
 				setbit(ctl.facilities, n);
 			break;
-		case 'V':
-			printf(UTIL_LINUX_VERSION);
-			return EXIT_SUCCESS;
 		case 'w':
 			ctl.follow = 1;
 			break;
@@ -1459,6 +1456,11 @@ int main(int argc, char *argv[])
 		case OPT_TIME_FORMAT:
 			ctl.time_fmt = which_time_format(optarg);
 			break;
+
+		case 'h':
+			usage();
+		case 'V':
+			print_version(EXIT_SUCCESS);
 		default:
 			errtryhelp(EXIT_FAILURE);
 		}
