@@ -71,6 +71,7 @@
 #include "all-io.h"
 #include "monotonic.h"
 #include "timeutils.h"
+#include "strutils.h"
 
 #include "debug.h"
 
@@ -104,6 +105,8 @@ struct script_control {
 	FILE *typescriptfp;	/* output file pointer */
 	char *tname;		/* timing file path */
 	FILE *timingfp;		/* timing file pointer */
+	uint64_t outsz;         /* current output file size */
+	uint64_t maxsz;		/* maximum output file size */
 	struct timeval oldtime;	/* previous write or command start time */
 	int master;		/* pseudoterminal master file descriptor */
 	int slave;		/* pseudoterminal slave file descriptor */
@@ -169,6 +172,7 @@ static void __attribute__((__noreturn__)) usage(void)
 		" -e, --return                  return exit code of the child process\n"
 		" -f, --flush                   run flush after each write\n"
 		"     --force                   use output file even when it is a link\n"
+		" -o, --output-limit <size>     terminate if output files exceed size\n"
 		" -q, --quiet                   be quiet\n"
 		" -t[<file>], --timing[=<file>] output timing data to stderr or to FILE\n"
 		), out);
@@ -178,6 +182,54 @@ static void __attribute__((__noreturn__)) usage(void)
 	exit(EXIT_SUCCESS);
 }
 
+static void typescript_message_start(const struct script_control *ctl, time_t *tvec)
+{
+	char buf[FORMAT_TIMESTAMP_MAX];
+	int cols = 0, lines = 0;
+	const char *tty = NULL, *term = NULL;
+
+	if (!ctl->typescriptfp)
+		return;
+
+	strtime_iso(tvec, ISO_TIMESTAMP, buf, sizeof(buf));
+
+	fprintf(ctl->typescriptfp, _("Script started on %s ["), buf);
+
+	if (ctl->isterm) {
+		get_terminal_dimension(&cols, &lines);
+		get_terminal_name(&tty, NULL, NULL);
+		get_terminal_type(&term);
+
+		if (term)
+			fprintf(ctl->typescriptfp, "TERM=\"%s\" ", term);
+		if (tty)
+			fprintf(ctl->typescriptfp, "TTY=\"%s\" ", tty);
+
+		fprintf(ctl->typescriptfp, "COLUMNS=\"%d\" LINES=\"%d\"", cols, lines);
+	} else
+		fprintf(ctl->typescriptfp, _("<not executed on terminal>"));
+
+	fputs("]\n", ctl->typescriptfp);
+}
+
+static void typescript_message_done(const struct script_control *ctl, int status, const char *msg)
+{
+	char buf[FORMAT_TIMESTAMP_MAX];
+	time_t tvec;
+
+	if (!ctl->typescriptfp)
+		return;
+
+	tvec = script_time((time_t *)NULL);
+
+	strtime_iso(&tvec, ISO_TIMESTAMP, buf, sizeof(buf));
+
+	if (msg)
+		fprintf(ctl->typescriptfp, _("\nScript done on %s [<%s>]\n"), buf, msg);
+	else
+		fprintf(ctl->typescriptfp, _("\nScript done on %s [COMMAND_EXIT_CODE=\"%d\"]\n"), buf, status);
+}
+
 static void die_if_link(const struct script_control *ctl)
 {
 	struct stat s;
@@ -215,14 +267,24 @@ static void enable_rawmode_tty(struct script_control *ctl)
 	tcsetattr(STDIN_FILENO, TCSANOW, &rtt);
 }
 
-static void __attribute__((__noreturn__)) done(struct script_control *ctl)
+static void __attribute__((__noreturn__)) done_log(struct script_control *ctl, const char *log_msg)
 {
+	int childstatus;
+
 	DBG(MISC, ul_debug("done!"));
 
 	restore_tty(ctl, TCSADRAIN);
 
-	if (!ctl->quiet && ctl->typescriptfp)
-		printf(_("Script done, file is %s\n"), ctl->fname);
+	if (WIFSIGNALED(ctl->childstatus))
+		childstatus = WTERMSIG(ctl->childstatus) + 0x80;
+	else
+		childstatus = WEXITSTATUS(ctl->childstatus);
+
+	if (ctl->typescriptfp) {
+		typescript_message_done(ctl, childstatus, log_msg);
+		if (!ctl->quiet)
+			printf(_("Script done, file is %s\n"), ctl->fname);
+	}
 #ifdef HAVE_LIBUTEMPTER
 	if (ctl->master >= 0)
 		utempter_remove_record(ctl->master);
@@ -234,13 +296,12 @@ static void __attribute__((__noreturn__)) done(struct script_control *ctl)
 	if (ctl->typescriptfp && close_stream(ctl->typescriptfp) != 0)
 		err(EXIT_FAILURE, "write failed: %s", ctl->fname);
 
-	if (ctl->rc_wanted) {
-		if (WIFSIGNALED(ctl->childstatus))
-			exit(WTERMSIG(ctl->childstatus) + 0x80);
-		else
-			exit(WEXITSTATUS(ctl->childstatus));
-	}
-	exit(EXIT_SUCCESS);
+	exit(ctl->rc_wanted ? childstatus : EXIT_SUCCESS);
+}
+
+static void __attribute__((__noreturn__)) done(struct script_control *ctl)
+{
+	done_log(ctl, NULL);
 }
 
 static void __attribute__((__noreturn__)) fail(struct script_control *ctl)
@@ -266,6 +327,8 @@ static void wait_for_child(struct script_control *ctl, int wait)
 static void write_output(struct script_control *ctl, char *obuf,
 			    ssize_t bytes)
 {
+	int timing_bytes = 0;
+
 	DBG(IO, ul_debug(" writing output"));
 
 	if (ctl->timing && ctl->timingfp) {
@@ -275,11 +338,13 @@ static void write_output(struct script_control *ctl, char *obuf,
 
 		gettime_monotonic(&now);
 		timersub(&now, &ctl->oldtime, &delta);
-		fprintf(ctl->timingfp, "%ld.%06ld %zd\n",
+		timing_bytes = fprintf(ctl->timingfp, "%ld.%06ld %zd\n",
 		        (long)delta.tv_sec, (long)delta.tv_usec, bytes);
 		if (ctl->flush)
 			fflush(ctl->timingfp);
 		ctl->oldtime = now;
+		if (timing_bytes < 0)
+			timing_bytes = 0;
 	}
 
 	DBG(IO, ul_debug("  writing to script file"));
@@ -299,6 +364,9 @@ static void write_output(struct script_control *ctl, char *obuf,
 		fail(ctl);
 	}
 
+	if (ctl->maxsz != 0)
+		ctl->outsz += bytes + timing_bytes;
+
 	DBG(IO, ul_debug("  writing output *done*"));
 }
 
@@ -350,7 +418,6 @@ static void handle_io(struct script_control *ctl, int fd, int *eof)
 {
 	char buf[BUFSIZ];
 	ssize_t bytes;
-
 	DBG(IO, ul_debug("%d FD active", fd));
 	*eof = 0;
 
@@ -379,10 +446,18 @@ static void handle_io(struct script_control *ctl, int fd, int *eof)
 		 * shell output that looks like double echoing */
 		fdatasync(ctl->master);
 
-	/* from command (master) to stdout */
+	/* from command (master) to stdout and log */
 	} else if (fd == ctl->master) {
 		DBG(IO, ul_debug(" master --> stdout %zd bytes", bytes));
 		write_output(ctl, buf, bytes);
+
+		/* check output limit */
+		if (ctl->maxsz != 0 && ctl->outsz >= ctl->maxsz) {
+			if (!ctl->quiet)
+				printf(_("Script terminated, max output file size %"PRIu64" exceeded.\n"), ctl->maxsz);
+			DBG(IO, ul_debug("output size %"PRIu64", exceeded limit %"PRIu64, ctl->outsz, ctl->maxsz));
+			done_log(ctl, _("max output size exceeded"));
+		}
 	}
 }
 
@@ -402,10 +477,15 @@ static void handle_signal(struct script_control *ctl, int fd)
 
 	switch (info.ssi_signo) {
 	case SIGCHLD:
-		DBG(SIGNAL, ul_debug(" get signal SIGCHLD"));
-		if (info.ssi_code == CLD_EXITED) {
+		DBG(SIGNAL, ul_debug(" get signal SIGCHLD [ssi_code=%d, ssi_status=%d]",
+							info.ssi_code, info.ssi_status));
+		if (info.ssi_code == CLD_EXITED
+		    || info.ssi_code == CLD_KILLED
+		    || info.ssi_code == CLD_DUMPED) {
 			wait_for_child(ctl, 0);
 			ctl->poll_timeout = 10;
+
+		/* In case of ssi_code is CLD_TRAPPED, CLD_STOPPED, or CLD_CONTINUED */
 		} else if (info.ssi_status == SIGSTOP && ctl->child) {
 			DBG(SIGNAL, ul_debug(" child stop by SIGSTOP -- stop parent too"));
 			kill(getpid(), SIGSTOP);
@@ -433,13 +513,13 @@ static void handle_signal(struct script_control *ctl, int fd)
 	default:
 		abort();
 	}
+	DBG(SIGNAL, ul_debug("signal handle on FD %d done", fd));
 }
 
 static void do_io(struct script_control *ctl)
 {
 	int ret, eof = 0;
 	time_t tvec = script_time((time_t *)NULL);
-	char buf[128];
 	enum {
 		POLLFD_SIGNAL = 0,
 		POLLFD_MASTER,
@@ -471,10 +551,9 @@ static void do_io(struct script_control *ctl)
 	}
 
 
-	if (ctl->typescriptfp) {
-		strtime_iso(&tvec, ISO_TIMESTAMP, buf, sizeof(buf));
-		fprintf(ctl->typescriptfp, _("Script started on %s\n"), buf);
-	}
+	if (ctl->typescriptfp)
+		typescript_message_start(ctl, &tvec);
+
 	gettime_monotonic(&ctl->oldtime);
 
 	while (!ctl->die) {
@@ -542,11 +621,6 @@ static void do_io(struct script_control *ctl)
 	if (!ctl->die)
 		wait_for_child(ctl, 1);
 
-	if (ctl->typescriptfp) {
-		tvec = script_time((time_t *)NULL);
-		strtime_iso(&tvec, ISO_TIMESTAMP, buf, sizeof(buf));
-		fprintf(ctl->typescriptfp, _("\nScript done on %s\n"), buf);
-	}
 	done(ctl);
 }
 
@@ -688,8 +762,7 @@ int main(int argc, char **argv)
 		.line = "/dev/ptyXX",
 #endif
 		.master = -1,
-		.poll_timeout = -1,
-		0
+		.poll_timeout = -1
 	};
 	int ch;
 
@@ -701,6 +774,7 @@ int main(int argc, char **argv)
 		{"return", no_argument, NULL, 'e'},
 		{"flush", no_argument, NULL, 'f'},
 		{"force", no_argument, NULL, FORCE_OPTION,},
+		{"output-limit", required_argument, NULL, 'o'},
 		{"quiet", no_argument, NULL, 'q'},
 		{"timing", optional_argument, NULL, 't'},
 		{"version", no_argument, NULL, 'V'},
@@ -719,11 +793,11 @@ int main(int argc, char **argv)
 	setlocale(LC_NUMERIC, "C");
 	bindtextdomain(PACKAGE, LOCALEDIR);
 	textdomain(PACKAGE);
-	atexit(close_stdout);
+	close_stdout_atexit();
 
 	script_init_debug();
 
-	while ((ch = getopt_long(argc, argv, "ac:efqt::Vh", longopts, NULL)) != -1)
+	while ((ch = getopt_long(argc, argv, "ac:efo:qt::Vh", longopts, NULL)) != -1)
 		switch (ch) {
 		case 'a':
 			ctl.append = 1;
@@ -740,6 +814,9 @@ int main(int argc, char **argv)
 		case FORCE_OPTION:
 			ctl.force = 1;
 			break;
+		case 'o':
+			ctl.maxsz = strtosize_or_err(optarg, _("failed to parse output limit size"));
+			break;
 		case 'q':
 			ctl.quiet = 1;
 			break;
@@ -748,13 +825,11 @@ int main(int argc, char **argv)
 				ctl.tname = optarg;
 			ctl.timing = 1;
 			break;
+
 		case 'V':
-			printf(UTIL_LINUX_VERSION);
-			exit(EXIT_SUCCESS);
-			break;
+			print_version(EXIT_SUCCESS);
 		case 'h':
 			usage();
-			break;
 		default:
 			errtryhelp(EXIT_FAILURE);
 		}
