@@ -20,10 +20,12 @@ TS_TOPDIR=$(cd ${0%/*} && pwd)
 SUBTESTS=
 EXCLUDETESTS=
 OPTS=
+SYSCOMMANDS=
 
 top_srcdir=
 top_builddir=
 paraller_jobs=1
+has_asan_opt=
 
 function num_cpus()
 {
@@ -59,15 +61,24 @@ while [ -n "$1" ]; do
 	--force |\
 	--fake |\
 	--memcheck-valgrind |\
-	--memcheck-asan |\
 	--nolocks |\
 	--show-diff |\
 	--verbose  |\
 	--skip-loopdevs |\
+	--noskip-commands |\
 	--parsable)
 		# these options are simply forwarded to the test scripts
 		OPTS="$OPTS $1"
 		;;
+	--memcheck-asan)
+		OPTS="$OPTS $1"
+		has_asan_opt="yes"
+		;;
+	--use-system-commands)
+		OPTS="$OPTS $1"
+		SYSCOMMANDS="yes"
+		;;
+
 	--nonroot)
 		if [ $(id -ru) -eq 0 ]; then
 			echo "Ignore util-linux test suite [non-root UID expected]."
@@ -98,18 +109,20 @@ while [ -n "$1" ]; do
 		echo "Usage: "
 		echo "  $(basename $0) [options] [<component> ...]"
 		echo "Options:"
-		echo "  --force              execute demanding tests"
-		echo "  --fake               do not run, setup tests only"
-		echo "  --memcheck-valgrind  run with valgrind"
-		echo "  --memcheck-asan      enable ASAN (requires ./configure --enable-asan)"
-		echo "  --nolocks            don't use flock to lock resources"
-		echo "  --verbose            verbose mode"
-		echo "  --show-diff          show diff from failed tests"
-		echo "  --nonroot            ignore test suite if user is root"
-		echo "  --srcdir=<path>      autotools top source directory"
-		echo "  --builddir=<path>    autotools top build directory"
-		echo "  --parallel=<num>     number of parallel test jobs, default: num cpus"
-		echo "  --exclude=<list>     exclude tests by list '<utilname>/<testname> ..'"
+		echo "  --force               execute demanding tests"
+		echo "  --fake                do not run, setup tests only"
+		echo "  --memcheck-valgrind   run with valgrind"
+		echo "  --memcheck-asan       enable ASAN (requires ./configure --enable-asan)"
+		echo "  --nolocks             don't use flock to lock resources"
+		echo "  --verbose             verbose mode"
+		echo "  --show-diff           show diff from failed tests"
+		echo "  --nonroot             ignore test suite if user is root"
+		echo "  --use-system-commands use PATH rather than builddir"
+		echo "  --noskip-commands     fail on missing commands"
+		echo "  --srcdir=<path>       autotools top source directory"
+		echo "  --builddir=<path>     autotools top build directory"
+		echo "  --parallel=<num>      number of parallel test jobs, default: num cpus"
+		echo "  --exclude=<list>      exclude tests by list '<utilname>/<testname> ..'"
 		echo
 		exit 1
 		;;
@@ -136,6 +149,14 @@ fi
 
 OPTS="$OPTS --srcdir=$top_srcdir --builddir=$top_builddir"
 
+# Auto-enable ASAN to avoid conflicts between tests and binaries
+if [ -z "$has_asan_opt" ]; then
+	asan=$(awk '/^ASAN_LDFLAGS/ { print $3 }' $top_builddir/Makefile)
+	if [ -n "$asan" ]; then
+		OPTS="$OPTS --memcheck-asan"
+	fi
+fi
+
 declare -a comps
 if [ -n "$SUBTESTS" ]; then
 	# selected tests only
@@ -148,7 +169,7 @@ if [ -n "$SUBTESTS" ]; then
 		fi
 	done
 else
-	if [ ! -f "$top_builddir/test_ttyutils" ]; then
+	if [ -z "$SYSCOMMANDS" -a ! -f "$top_builddir/test_ttyutils" ]; then
 		echo "Tests not compiled! Run 'make check' to fix the problem."
 		exit 1
 	fi
@@ -187,6 +208,8 @@ echo
 printf "%13s: %-30s    " "kernel" "$(uname -r)"
 echo
 echo
+echo "      options: $(echo $OPTS | sed 's/ / \\\n               /g')"
+echo
 
 if [ "$paraller_jobs" -ne 1 ]; then
 	tmp=$paraller_jobs
