@@ -94,7 +94,9 @@ struct column_control {
 
 	unsigned int greedy :1,
 		     json :1,
-		     header_repeat :1;
+		     header_repeat :1,
+		     tab_empty_lines :1,	/* --table-empty-lines */
+		     tab_noheadings :1;
 };
 
 static size_t width(const wchar_t *str)
@@ -219,6 +221,7 @@ static void init_table(struct column_control *ctl)
 			scols_table_new_column(ctl->tab, *name, 0, 0);
 		if (ctl->header_repeat)
 			scols_table_enable_header_repeat(ctl->tab, 1);
+		scols_table_enable_noheadings(ctl->tab, !!ctl->tab_noheadings);
 	} else
 		scols_table_enable_noheadings(ctl->tab, 1);
 }
@@ -275,13 +278,38 @@ static void apply_columnflag_from_list(struct column_control *ctl, const char *l
 {
 	char **all = split_or_error(list, errmsg);
 	char **one;
+	int unnamed = 0;
 
 	STRV_FOREACH(one, all) {
-		struct libscols_column *cl = string_to_column(ctl, *one);
+		struct libscols_column *cl;
+
+		if (flag == SCOLS_FL_HIDDEN && strcmp(*one, "-") == 0) {
+			unnamed = 1;
+			continue;
+		}
+		cl = string_to_column(ctl, *one);
 		if (cl)
 			column_set_flag(cl, flag);
 	}
 	strv_free(all);
+
+	/* apply flag to all columns without name */
+	if (unnamed) {
+		struct libscols_iter *itr;
+		struct libscols_column *cl;
+
+		itr = scols_new_iter(SCOLS_ITER_FORWARD);
+		if (!itr)
+			err_oom();
+
+		while (scols_table_next_column(ctl->tab, itr, &cl) == 0) {
+			struct libscols_cell *ce = scols_column_get_header(cl);
+
+			if (ce == NULL ||  scols_cell_get_data(ce) == NULL)
+				column_set_flag(cl, flag);
+		}
+		scols_free_iter(itr);
+	}
 }
 
 static void reorder_table(struct column_control *ctl)
@@ -304,6 +332,9 @@ static void reorder_table(struct column_control *ctl)
 		scols_table_move_column(ctl->tab, last, wanted[i]);
 		last = wanted[i];
 	}
+
+	free(wanted);
+	strv_free(order);
 }
 
 static void create_tree(struct column_control *ctl)
@@ -420,6 +451,7 @@ static int add_line_to_table(struct column_control *ctl, wchar_t *wcs)
 			if (!ln)
 				err(EXIT_FAILURE, _("failed to allocate output line"));
 		}
+
 		data = wcs_to_mbs(wcdata);
 		if (!data)
 			err(EXIT_FAILURE, _("failed to allocate output data"));
@@ -432,6 +464,17 @@ static int add_line_to_table(struct column_control *ctl, wchar_t *wcs)
 	return 0;
 }
 
+static int add_emptyline_to_table(struct column_control *ctl)
+{
+	if (!ctl->tab)
+		init_table(ctl);
+
+	if (!scols_table_new_line(ctl->tab, NULL))
+		err(EXIT_FAILURE, _("failed to allocate output line"));
+
+	return 0;
+}
+
 static int read_input(struct column_control *ctl, FILE *fp)
 {
 	char *buf = NULL;
@@ -439,6 +482,7 @@ static int read_input(struct column_control *ctl, FILE *fp)
 	size_t maxents = 0;
 	int rc = 0;
 
+	/* Read input */
 	do {
 		char *str, *p;
 		wchar_t *wcs = NULL;
@@ -455,16 +499,19 @@ static int read_input(struct column_control *ctl, FILE *fp)
 			if (p)
 				*p = '\0';
 		}
-		if (!str || !*str)
+		if (!str || !*str) {
+			if (ctl->mode == COLUMN_MODE_TABLE && ctl->tab_empty_lines)
+				add_emptyline_to_table(ctl);
 			continue;
+		}
 
-		wcs = mbs_to_wcs(str);
+		wcs = mbs_to_wcs(buf);
 		if (!wcs) {
 			/*
 			 * Convert broken sequences to \x<hex> and continue.
 			 */
 			size_t tmpsz = 0;
-			char *tmp = mbs_invalid_encode(str, &tmpsz);
+			char *tmp = mbs_invalid_encode(buf, &tmpsz);
 
 			if (!tmp)
 				err(EXIT_FAILURE, _("read failed"));
@@ -491,6 +538,9 @@ static int read_input(struct column_control *ctl, FILE *fp)
 				ctl->maxlength = len;
 			ctl->nents++;
 			break;
+		default:
+			free(wcs);
+			break;
 		}
 	} while (rc == 0);
 
@@ -583,11 +633,13 @@ static void __attribute__((__noreturn__)) usage(void)
 	fputs(_(" -O, --table-order <columns>      specify order of output columns\n"), out);
 	fputs(_(" -N, --table-columns <names>      comma separated columns names\n"), out);
 	fputs(_(" -E, --table-noextreme <columns>  don't count long text from the columns to column width\n"), out);
+	fputs(_(" -d, --table-noheadings           don't print header\n"), out);
 	fputs(_(" -e, --table-header-repeat        repeat header for each page\n"), out);
 	fputs(_(" -H, --table-hide <columns>       don't print the columns\n"), out);
 	fputs(_(" -R, --table-right <columns>      right align text in these columns\n"), out);
 	fputs(_(" -T, --table-truncate <columns>   truncate text in the columns when necessary\n"), out);
 	fputs(_(" -W, --table-wrap <columns>       wrap text in the columns when necessary\n"), out);
+	fputs(_(" -L, --table-empty-lines          don't ignore empty lines\n"), out);
 	fputs(_(" -J, --json                       use JSON output format for table\n"), out);
 
 	fputs(USAGE_SEPARATOR, out);
@@ -634,10 +686,12 @@ int main(int argc, char **argv)
 		{ "table-hide",          required_argument, NULL, 'H' },
 		{ "table-name",          required_argument, NULL, 'n' },
 		{ "table-noextreme",     required_argument, NULL, 'E' },
+		{ "table-noheadings",    no_argument,       NULL, 'd' },
 		{ "table-order",         required_argument, NULL, 'O' },
 		{ "table-right",         required_argument, NULL, 'R' },
 		{ "table-truncate",      required_argument, NULL, 'T' },
 		{ "table-wrap",          required_argument, NULL, 'W' },
+		{ "table-empty-lines",   no_argument,       NULL, 'L' },
 		{ "table-header-repeat", no_argument,       NULL, 'e' },
 		{ "tree",                required_argument, NULL, 'r' },
 		{ "tree-id",             required_argument, NULL, 'i' },
@@ -655,12 +709,12 @@ int main(int argc, char **argv)
 	setlocale(LC_ALL, "");
 	bindtextdomain(PACKAGE, LOCALEDIR);
 	textdomain(PACKAGE);
-	atexit(close_stdout);
+	close_stdout_atexit();
 
 	ctl.output_separator = "  ";
 	ctl.input_separator = mbs_to_wcs("\t ");
 
-	while ((c = getopt_long(argc, argv, "c:E:eH:hi:JN:n:O:o:p:R:r:s:T:tVW:x", longopts, NULL)) != -1) {
+	while ((c = getopt_long(argc, argv, "c:dE:eH:hi:JLN:n:O:o:p:R:r:s:T:tVW:x", longopts, NULL)) != -1) {
 
 		err_exclusive_options(c, longopts, excl, excl_st);
 
@@ -668,6 +722,9 @@ int main(int argc, char **argv)
 		case 'c':
 			ctl.termwidth = strtou32_or_err(optarg, _("invalid columns argument"));
 			break;
+		case 'd':
+			ctl.tab_noheadings = 1;
+			break;
 		case 'E':
 			ctl.tab_colnoextrem = optarg;
 			break;
@@ -677,9 +734,6 @@ int main(int argc, char **argv)
 		case 'H':
 			ctl.tab_colhide = optarg;
 			break;
-		case 'h':
-			usage();
-			break;
 		case 'i':
 			ctl.tree_id = optarg;
 			break;
@@ -687,6 +741,9 @@ int main(int argc, char **argv)
 			ctl.json = 1;
 			ctl.mode = COLUMN_MODE_TABLE;
 			break;
+		case 'L':
+			ctl.tab_empty_lines = 1;
+			break;
 		case 'N':
 			ctl.tab_colnames = split_or_error(optarg, _("failed to parse column names"));
 			break;
@@ -719,15 +776,17 @@ int main(int argc, char **argv)
 		case 't':
 			ctl.mode = COLUMN_MODE_TABLE;
 			break;
-		case 'V':
-			printf(UTIL_LINUX_VERSION);
-			return EXIT_SUCCESS;
 		case 'W':
 			ctl.tab_colwrap = optarg;
 			break;
 		case 'x':
 			ctl.mode = COLUMN_MODE_FILLROWS;
 			break;
+
+		case 'h':
+			usage();
+		case 'V':
+			print_version(EXIT_SUCCESS);
 		default:
 			errtryhelp(EXIT_FAILURE);
 		}
